@@ -80,7 +80,6 @@ Boston, MA 02111-1307, USA.  */
 #ifdef REGION_CACHE_NEEDS_WORK
 #include "region-cache.h"
 #endif
-#include "symeval.h"
 #include "syntax.h"
 #include "sysdep.h"	/* for getwd */
 #include "window.h"
@@ -98,7 +97,7 @@ struct buffer *current_buffer;	/* the current buffer */
 Lisp_Object Vbuffer_defaults;
 
 /* This structure marks which slots in a buffer have corresponding
-   default values in buffer_defaults.
+   default values in Vbuffer_defaults.
    Each such slot has a nonzero value in this structure.
    The value has only one nonzero bit.
 
@@ -108,7 +107,7 @@ Lisp_Object Vbuffer_defaults;
 
    If a slot in this structure is 0, then there is a DEFVAR_BUFFER_LOCAL
    for the slot, but there is no default value for it; the corresponding
-   slot in buffer_defaults is not used except to initialize newly-created
+   slot in Vbuffer_defaults is not used except to initialize newly-created
    buffers.
 
    If a slot is -1, then there is a DEFVAR_BUFFER_LOCAL for it
@@ -121,12 +120,15 @@ Lisp_Object Vbuffer_defaults;
 
    If a slot is -3, then there is no DEFVAR_BUFFER_LOCAL for it but
    there is a default which is used to initialize newly-creation
-   buffers and as a reset-value when local-vars are killed.
-
-
-   */
+   buffers and as a reset-value when local-vars are killed.  */
 struct buffer buffer_local_flags;
 
+/* This is the initial (startup) directory, as used for the *scratch* buffer.
+   We're making this a global to make others aware of the startup directory.
+   `initial_directory' is stored in external format.
+ */
+char initial_directory[MAXPATHLEN+1];
+
 /* This structure holds the names of symbols whose values may be
    buffer-local.  It is indexed and accessed in the same way as the above. */
 static Lisp_Object Vbuffer_local_symbols;
@@ -207,19 +209,6 @@ int find_file_use_truenames;
 
 static void reset_buffer_local_variables (struct buffer *, int first_time);
 static void nuke_all_buffer_slots (struct buffer *b, Lisp_Object zap);
-static Lisp_Object mark_buffer (Lisp_Object, void (*) (Lisp_Object));
-static void print_buffer (Lisp_Object, Lisp_Object, int);
-/* We do not need a finalize method to handle a buffer's children list
-   because all buffers have `kill-buffer' applied to them before
-   they disappear, and the children removal happens then. */
-DEFINE_LRECORD_IMPLEMENTATION ("buffer", buffer,
-                               mark_buffer, print_buffer, 0, 0, 0,
-			       struct buffer);
-
-#ifdef ENERGIZE
-extern void mark_energize_buffer_data (struct buffer *b,
-				       void (*markobj) (Lisp_Object));
-#endif
 
 Lisp_Object
 make_buffer (struct buffer *buf)
@@ -243,26 +232,14 @@ mark_buffer (Lisp_Object obj, void (*markobj) (Lisp_Object))
 #include "bufslots.h"
 #undef MARKED_SLOT
 
-#ifdef ENERGIZE
-  mark_energize_buffer_data (XBUFFER (obj), markobj);
-#endif
-
   ((markobj) (buf->extent_info));
 
   /* Don't mark normally through the children slot.
-     (Actually, in this case, it doesn't matter.)
-   */
-  mark_conses_in_list (buf->indirect_children);
+     (Actually, in this case, it doesn't matter.)  */
+  if (! EQ (buf->indirect_children, Qnull_pointer))
+    mark_conses_in_list (buf->indirect_children);
 
-  if (buf->base_buffer)
-    {
-      Lisp_Object base_buf_obj = Qnil;
-
-      XSETBUFFER (base_buf_obj, buf->base_buffer);
-      return base_buf_obj;
-    }
-  else
-    return Qnil;
+  return buf->base_buffer ? make_buffer (buf->base_buffer) : Qnil;
 }
 
 static void
@@ -292,25 +269,27 @@ print_buffer (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
     }
 }
 
+/* We do not need a finalize method to handle a buffer's children list
+   because all buffers have `kill-buffer' applied to them before
+   they disappear, and the children removal happens then. */
+DEFINE_LRECORD_IMPLEMENTATION ("buffer", buffer,
+                               mark_buffer, print_buffer, 0, 0, 0,
+			       struct buffer);
 
 DEFUN ("bufferp", Fbufferp, 1, 1, 0, /*
-T if OBJECT is an editor buffer.
+Return t if OBJECT is an editor buffer.
 */
        (object))
 {
-  if (BUFFERP (object))
-    return Qt;
-  return Qnil;
+  return BUFFERP (object) ? Qt : Qnil;
 }
 
 DEFUN ("buffer-live-p", Fbuffer_live_p, 1, 1, 0, /*
-T if OBJECT is an editor buffer that has not been deleted.
+Return t if OBJECT is an editor buffer that has not been deleted.
 */
        (object))
 {
-  if (BUFFERP (object) && BUFFER_LIVE_P (XBUFFER (object)))
-    return Qt;
-  return Qnil;
+  return BUFFERP (object) && BUFFER_LIVE_P (XBUFFER (object)) ? Qt : Qnil;
 }
 
 static void
@@ -341,8 +320,6 @@ returned instead.
 Lisp_Object
 get_buffer (Lisp_Object name, int error_if_deleted_or_does_not_exist)
 {
-  Lisp_Object buf;
-
   if (BUFFERP (name))
     {
       if (!BUFFER_LIVE_P (XBUFFER (name)))
@@ -355,6 +332,7 @@ get_buffer (Lisp_Object name, int error_if_deleted_or_does_not_exist)
     }
   else
     {
+      Lisp_Object buf;
       struct gcpro gcpro1;
 
       CHECK_STRING (name);
@@ -497,7 +475,7 @@ the search will still be done on `buffer-file-name'.
       NUNGCPRO;
     }
 
-  for (tail = Vbuffer_alist; CONSP (tail); tail = XCDR (tail))
+  LIST_LOOP (tail, Vbuffer_alist)
     {
       buf = Fcdr (XCAR (tail));
       if (!BUFFERP (buf)) continue;
@@ -693,6 +671,7 @@ If BASE is an indirect buffer itself, the base buffer for that buffer
 }
 #endif /* 0 */
 
+
 
 static void
 reset_buffer_local_variables (struct buffer *b, int first_time)
@@ -712,6 +691,28 @@ reset_buffer_local_variables (struct buffer *b, int first_time)
   }
 #include "bufslots.h"
 #undef MARKED_SLOT
+#if 0
+#define STRING256_P(obj) \
+  (STRINGP (obj) && XSTRING_CHAR_LENGTH (obj) == 256)
+  /* If the standard case table has been altered and invalidated,
+     fix up its insides first.  */
+  if (!(STRING256_P(Vascii_upcase_table) &&
+	STRING256_P(Vascii_canon_table) &&
+	STRING256_P(Vascii_eqv_table)))
+    {
+      Fset_standard_case_table (Vascii_downcase_table);
+    }
+  b->downcase_table = Vascii_downcase_table;
+  b->upcase_table = Vascii_upcase_table;
+  b->case_canon_table = Vascii_canon_table;
+  b->case_eqv_table = Vascii_eqv_table;
+#ifdef MULE
+  b->mirror_downcase_table = Vmirror_ascii_downcase_table;
+  b->mirror_upcase_table = Vmirror_ascii_upcase_table;
+  b->mirror_case_canon_table = Vmirror_ascii_canon_table;
+  b->mirror_case_eqv_table = Vmirror_ascii_eqv_table;
+#endif
+#endif
 }
 
 
@@ -798,13 +799,8 @@ If BUFFER is not indirect, return nil.
        (buffer))
 {
   struct buffer *buf = decode_buffer (buffer, 0);
-  struct buffer *base = buf->base_buffer;
-  Lisp_Object base_buffer = Qnil;
 
-  if (! base)
-    return Qnil;
-  XSETBUFFER (base_buffer, base);
-  return base_buffer;
+  return buf->base_buffer ? make_buffer (buf->base_buffer) : Qnil;
 }
 
 DEFUN ("buffer-indirect-children", Fbuffer_indirect_children, 0, 1, 0, /*
@@ -958,12 +954,6 @@ as BUFFER means use current buffer.
   /* This function can GC */
   struct buffer *buf = decode_buffer (buffer, 0);
 
-#ifdef ENERGIZE
-  Lisp_Object starting_flag =
-    (BUF_SAVE_MODIFF (buf) < BUF_MODIFF (buf)) ? Qt : Qnil;
-  Lisp_Object argument_flag = (NILP (flag)) ? Qnil : Qt;
-#endif
-
 #ifdef CLASH_DETECTION
   /* If buffer becoming modified, lock the file.
      If buffer becoming unmodified, unlock the file.  */
@@ -999,22 +989,6 @@ as BUFFER means use current buffer.
   BUF_SAVE_MODIFF (buf) = NILP (flag) ? BUF_MODIFF (buf) : 0;
   MARK_MODELINE_CHANGED;
 
-#ifdef ENERGIZE
-  /* don't send any notification if we are "setting" the modification bit
-     to be the same as it already was */
-  if (!EQ (starting_flag, argument_flag))
-    {
-      extern Lisp_Object Qenergize_buffer_modified_hook;
-      int count = specpdl_depth ();
-      record_unwind_protect (Fset_buffer, Fcurrent_buffer ());
-      set_buffer_internal (buf);
-      va_run_hook_with_args (Qenergize_buffer_modified_hook, 3,
-			     flag, make_int (BUF_BEG (buf)),
-			     make_int (BUF_Z (buf)));
-      unbind_to (count, Qnil);
-    }
-#endif /* ENERGIZE */
-
   return flag;
 }
 
@@ -1169,7 +1143,7 @@ No argument or nil as argument means do this for the current buffer.
 */
        (buffer))
 {
-  /* Allowing nil is an Emacs-ism */
+  /* Allowing nil is an RMSism */
   struct buffer *real_buf = decode_buffer (buffer, 1);
   real_buf->undo_list = Qt;
   return Qnil;
@@ -1181,7 +1155,7 @@ No argument or nil as argument means do this for the current buffer.
 */
        (buffer))
 {
-  /* Allowing nil is an Emacs-ism */
+  /* Allowing nil is an RMSism */
   struct buffer *real_buf = decode_buffer (buffer, 1);
   if (EQ (real_buf->undo_list, Qt))
     real_buf->undo_list = Qnil;
@@ -1235,10 +1209,6 @@ with `delete-process'.
   if (EQ (buf, Vecho_area_buffer))
     return Qnil;
 
-  /* Or the special invisible internal prin1 buffer. */
-  if (EQ (buf, Vprin1_to_string_buffer))
-    return Qnil;
-
   /* Query if the buffer is still modified.  */
   if (INTERACTIVE && !NILP (b->filename)
       && BUF_MODIFF (b) > BUF_SAVE_MODIFF (b))
@@ -1271,9 +1241,7 @@ with `delete-process'.
 
       /* First run the query functions; if any query is answered no,
          don't kill the buffer.  */
-      for (tail = Vkill_buffer_query_functions;
-           !NILP (tail);
-           tail = Fcdr (tail))
+      EXTERNAL_LIST_LOOP (tail, Vkill_buffer_query_functions)
 	{
 	  if (NILP (call0 (Fcar (tail))))
 	    {
@@ -1608,17 +1576,16 @@ set_buffer_internal (struct buffer *b)
 }
 
 DEFUN ("set-buffer", Fset_buffer, 1, 1, 0, /*
-Make the buffer BUFNAME current for editing operations.
-BUFNAME may be a buffer or the name of an existing buffer.
+Make the buffer BUFFER current for editing operations.
+BUFFER may be a buffer or the name of an existing buffer.
 See also `save-excursion' when you want to make a buffer current temporarily.
 This function does not display the buffer, so its effect ends
 when the current command terminates.
 Use `switch-to-buffer' or `pop-to-buffer' to switch buffers permanently.
 */
-       (bufname))
+       (buffer))
 {
-  Lisp_Object buffer;
-  buffer = get_buffer (bufname, 0);
+  buffer = get_buffer (buffer, 0);
   if (NILP (buffer))
     error ("Selecting deleted or non-existent buffer");
   set_buffer_internal (XBUFFER (buffer));
@@ -1637,7 +1604,7 @@ however, are not checked.) END defaults to the value of START.
 
 If START and END are equal, the range checked is [START, END] (i.e.
 closed on both ends); otherwise, the range checked is (START, END)
-(open on both ends), except that extents that lie completely within
+\(open on both ends), except that extents that lie completely within
 [START, END] are not checked.  See `extent-in-region-p' for a fuller
 discussion.
 */
@@ -1742,7 +1709,7 @@ BUFFER defaults to the current buffer if omitted.
   /* Prevent warnings, or suspension of auto saving, that would happen
      if future size is less than past size.  Use of erase-buffer
      implies that the future text is not really related to the past text.  */
-  b->save_length = Qzero;
+  b->saved_size = Qzero;
 
   zmacs_region_stays = 0;
   return Qnil;
@@ -1797,18 +1764,15 @@ struct buffer_stats
   int other;
 };
 
-static int
+static size_t
 compute_buffer_text_usage (struct buffer *b, struct overhead_stats *ovstats)
 {
-  int malloc_use;
-  int was_requested;
-  int gap;
+  int was_requested = b->text->z - 1;
+  size_t gap = b->text->gap_size + b->text->end_gap_size;
+  size_t malloc_use = malloced_storage_size (b->text->beg, was_requested + gap, 0);
 
-  was_requested = b->text->z - 1;
-  gap = b->text->gap_size + b->text->end_gap_size;
-  malloc_use = malloced_storage_size (b->text->beg, was_requested + gap, 0);
-  ovstats->gap_overhead += gap;
-  ovstats->was_requested += was_requested;
+  ovstats->gap_overhead    += gap;
+  ovstats->was_requested   += was_requested;
   ovstats->malloc_overhead += malloc_use - (was_requested + gap);
   return malloc_use;
 }
@@ -1817,22 +1781,22 @@ static void
 compute_buffer_usage (struct buffer *b, struct buffer_stats *stats,
 		      struct overhead_stats *ovstats)
 {
-  memset (stats, 0, sizeof (*stats));
-  stats->other += malloced_storage_size (b, sizeof (struct buffer), ovstats);
-  stats->text += compute_buffer_text_usage (b, ovstats);
+  xzero (*stats);
+  stats->other   += malloced_storage_size (b, sizeof (*b), ovstats);
+  stats->text    += compute_buffer_text_usage   (b, ovstats);
   stats->markers += compute_buffer_marker_usage (b, ovstats);
   stats->extents += compute_buffer_extent_usage (b, ovstats);
 }
 
 DEFUN ("buffer-memory-usage", Fbuffer_memory_usage, 1, 1, 0, /*
 Return stats about the memory usage of buffer BUFFER.
-The values returned are in the form an alist of usage types and byte
+The values returned are in the form of an alist of usage types and byte
 counts.  The byte counts attempt to encompass all the memory used
 by the buffer (separate from the memory logically associated with a
 buffer or frame), including internal structures and any malloc()
 overhead associated with them.  In practice, the byte counts are
 underestimated because certain memory usage is very hard to determine
-(e.g. the amount of memory used inside the Xt library or inside the
+\(e.g. the amount of memory used inside the Xt library or inside the
 X server) and because there is other stuff that might logically
 be associated with a window, buffer, or frame (e.g. window configurations,
 glyphs) but should not obviously be included in the usage counts.
@@ -1847,24 +1811,23 @@ represents all the memory concerned.
 {
   struct buffer_stats stats;
   struct overhead_stats ovstats;
+  Lisp_Object val = Qnil;
 
   CHECK_BUFFER (buffer); /* dead buffers should be allowed, no? */
-  memset (&ovstats, 0, sizeof (ovstats));
+  xzero (ovstats);
   compute_buffer_usage (XBUFFER (buffer), &stats, &ovstats);
 
-  return nconc2 (list4 (Fcons (Qtext, make_int (stats.text)),
-			Fcons (Qmarkers, make_int (stats.markers)),
-			Fcons (Qextents, make_int (stats.extents)),
-			Fcons (Qother, make_int (stats.other))),
-		 list5 (Qnil,
-			Fcons (Qactually_requested,
-			       make_int (ovstats.was_requested)),
-			Fcons (Qmalloc_overhead,
-			       make_int (ovstats.malloc_overhead)),
-			Fcons (Qgap_overhead,
-			       make_int (ovstats.malloc_overhead)),
-			Fcons (Qdynarr_overhead,
-			       make_int (ovstats.dynarr_overhead))));
+  val = acons (Qtext,    make_int (stats.text),    val);
+  val = acons (Qmarkers, make_int (stats.markers), val);
+  val = acons (Qextents, make_int (stats.extents), val);
+  val = acons (Qother,   make_int (stats.other),   val);
+  val = Fcons (Qnil, val);
+  val = acons (Qactually_requested, make_int (ovstats.was_requested),   val);
+  val = acons (Qmalloc_overhead,    make_int (ovstats.malloc_overhead), val);
+  val = acons (Qgap_overhead,       make_int (ovstats.gap_overhead),    val);
+  val = acons (Qdynarr_overhead,    make_int (ovstats.dynarr_overhead), val);
+
+  return Fnreverse (val);
 }
 
 #endif /* MEMORY_USAGE_STATS */
@@ -2096,7 +2059,7 @@ List of functions called with no args to query before killing a buffer.
    makes debugging unbelievably more bearable) */
 #define DEFVAR_BUFFER_LOCAL(lname, field_name) do {			\
 static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
-  = { { { { lrecord_symbol_value_forward },				\
+  = { { { symbol_value_forward_lheader_initializer,			\
      (struct lcrecord_header *) &(buffer_local_flags.field_name), 69 },	\
       SYMVAL_CURRENT_BUFFER_FORWARD }, 0 };				\
       defvar_buffer_local ((lname), &I_hate_C);				\
@@ -2104,7 +2067,7 @@ static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
 
 #define DEFVAR_BUFFER_LOCAL_MAGIC(lname, field_name, magicfun) do {	\
 static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
-  = { { { { lrecord_symbol_value_forward },				\
+  = { { { symbol_value_forward_lheader_initializer,			\
      (struct lcrecord_header *) &(buffer_local_flags.field_name), 69 },	\
       SYMVAL_CURRENT_BUFFER_FORWARD }, magicfun };			\
       defvar_buffer_local ((lname), &I_hate_C);				\
@@ -2112,7 +2075,7 @@ static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
 
 #define DEFVAR_CONST_BUFFER_LOCAL(lname, field_name) do {		\
 static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
-  = { { { { lrecord_symbol_value_forward },				\
+  = { { { symbol_value_forward_lheader_initializer,			\
      (struct lcrecord_header *) &(buffer_local_flags.field_name), 69 },	\
       SYMVAL_CONST_CURRENT_BUFFER_FORWARD }, 0 };			\
       defvar_buffer_local ((lname), &I_hate_C);				\
@@ -2120,7 +2083,7 @@ static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
 
 #define DEFVAR_CONST_BUFFER_LOCAL_MAGIC(lname, field_name, magicfun) do{\
 static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
-  = { { { { lrecord_symbol_value_forward },				\
+  = { { { symbol_value_forward_lheader_initializer,			\
      (struct lcrecord_header *) &(buffer_local_flags.field_name), 69 },	\
       SYMVAL_CONST_CURRENT_BUFFER_FORWARD }, magicfun };		\
       defvar_buffer_local ((lname), &I_hate_C);				\
@@ -2143,7 +2106,7 @@ defvar_buffer_local (CONST char *namestring,
  *  by make-docfile */
 #define DEFVAR_BUFFER_DEFAULTS(lname, field_name) do {			\
 static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
-  = { { { { lrecord_symbol_value_forward },				\
+  = { { { symbol_value_forward_lheader_initializer,			\
      (struct lcrecord_header *) &(buffer_local_flags.field_name), 69 },	\
       SYMVAL_DEFAULT_BUFFER_FORWARD }, 0 };				\
       defvar_mumble ((lname), &I_hate_C, sizeof (I_hate_C));		\
@@ -2151,7 +2114,7 @@ static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
 
 #define DEFVAR_BUFFER_DEFAULTS_MAGIC(lname, field_name, magicfun) do {	\
 static CONST_IF_NOT_DEBUG struct symbol_value_forward I_hate_C		\
-  = { { { { lrecord_symbol_value_forward },				\
+  = { { { symbol_value_forward_lheader_initializer,			\
      (struct lcrecord_header *) &(buffer_local_flags.field_name), 69 },	\
       SYMVAL_DEFAULT_BUFFER_FORWARD }, magicfun };			\
       defvar_mumble ((lname), &I_hate_C, sizeof (I_hate_C));		\
@@ -2199,13 +2162,11 @@ complex_vars_of_buffer (void)
   defs->mirror_upcase_table = Vmirror_ascii_upcase_table;
   defs->mirror_case_canon_table = Vmirror_ascii_canon_table;
   defs->mirror_case_eqv_table = Vmirror_ascii_eqv_table;
+  defs->category_table = Vstandard_category_table;
 #endif /* MULE */
   defs->syntax_table = Vstandard_syntax_table;
   defs->mirror_syntax_table =
     XCHAR_TABLE (Vstandard_syntax_table)->mirror_table;
-#ifdef MULE
-  defs->category_table = Vstandard_category_table;
-#endif
   defs->modeline_format = build_string ("%-");  /* reset in loaddefs.el */
   defs->case_fold_search = Qt;
   defs->selective_display_ellipses = Qt;
@@ -2213,12 +2174,15 @@ complex_vars_of_buffer (void)
   defs->ctl_arrow = Qt;
   defs->fill_column = make_int (70);
   defs->left_margin = Qzero;
-  defs->save_length = Qzero;       /* lisp code wants int-or-nil */
+  defs->saved_size = Qzero;	/* lisp code wants int-or-nil */
   defs->modtime = 0;
   defs->auto_save_modified = 0;
   defs->auto_save_failure_time = -1;
   defs->invisibility_spec = Qt;
 
+  defs->indirect_children = Qnil;
+  syms->indirect_children = Qnil;
+
   {
     /*  0 means var is always local.  Default used only at creation.
      * -1 means var is always local.  Default used only at reset and
@@ -2243,7 +2207,7 @@ complex_vars_of_buffer (void)
     buffer_local_flags.filename = always_local_no_default;
     buffer_local_flags.directory = always_local_no_default;
     buffer_local_flags.backed_up = always_local_no_default;
-    buffer_local_flags.save_length = always_local_no_default;
+    buffer_local_flags.saved_size = always_local_no_default;
     buffer_local_flags.auto_save_file_name = always_local_no_default;
     buffer_local_flags.read_only = always_local_no_default;
 
@@ -2285,9 +2249,8 @@ complex_vars_of_buffer (void)
 #ifdef REGION_CACHE_NEEDS_WORK
     buffer_local_flags.cache_long_line_scans = make_int (0x2000);
 #endif
-    buffer_local_flags.buffer_file_type = make_int (0x4000);
-#ifdef MULE
-    buffer_local_flags.buffer_file_coding_system = make_int (0x8000);
+#ifdef FILE_CODING
+    buffer_local_flags.buffer_file_coding_system = make_int (0x4000);
 #endif
 
     /* #### Warning, 0x4000000 (that's six zeroes) is the largest number
@@ -2341,12 +2304,6 @@ This is the same as (default-value 'tab-width).
   DEFVAR_BUFFER_DEFAULTS ("default-case-fold-search", case_fold_search /*
 Default value of `case-fold-search' for buffers that don't override it.
 This is the same as (default-value 'case-fold-search).
-*/ );
-
-  DEFVAR_BUFFER_DEFAULTS ("default-buffer-file-type", buffer_file_type /*
-Default file type for buffers that do not override it.
-This is the same as (default-value 'buffer-file-type).
-The file type is nil for text, t for binary.
 */ );
 
   DEFVAR_BUFFER_LOCAL ("modeline-format", modeline_format /*
@@ -2391,7 +2348,6 @@ A string is printed verbatim in the modeline except for %-constructs:
   %P -- print percent of buffer above bottom of window, perhaps plus Top,
         or print Bottom or All.
   %n -- print Narrow if appropriate.
-  %t -- Under MS-DOS, print T if files is text, B if binary.
   %C -- under XEmacs/mule, print the mnemonic for `buffer-file-coding-system'.
   %[ -- print one [ for each recursive editing level.  %] similar.
   %% -- print %.                %- -- print infinitely many dashes.
@@ -2486,14 +2442,7 @@ Name of default directory of current buffer.  Should end with slash.
 Each buffer has its own value of this variable.
 */ );
 
-  DEFVAR_BUFFER_LOCAL ("buffer-file-type", buffer_file_type /*
-    "Non-nil if the visited file is a binary file.
-This variable is meaningful on MS-DOS and Windows NT.
-On those systems, it is automatically local in every buffer.
-On other systems, this variable is normally always nil.
-*/ );
-
-#ifdef MULE
+#ifdef FILE_CODING
   DEFVAR_BUFFER_DEFAULTS ("default-buffer-file-coding-system", buffer_file_coding_system /*
 Default value of `buffer-file-coding-system' for buffers that do not override it.
 This is the same as (default-value 'buffer-file-coding-system).
@@ -2542,8 +2491,9 @@ you wish to unilaterally specify the coding system used for one
 particular operation, you should bind the variable
 `coding-system-for-read' rather than changing the other two
 variables just mentioned, which are intended to be used for
-global environment specification.  */ );
-#endif /* MULE */
+global environment specification.
+*/ );
+#endif
 
   DEFVAR_BUFFER_LOCAL ("auto-fill-function", auto_fill_function /*
 Function called (if non-nil) to perform auto-fill.
@@ -2593,7 +2543,7 @@ Backing up is done before the first time the file is saved.
 Each buffer has its own value of this variable.
 */ );
 
-  DEFVAR_BUFFER_LOCAL ("buffer-saved-size", save_length /*
+  DEFVAR_BUFFER_LOCAL ("buffer-saved-size", saved_size /*
 Length of current buffer when last read in, saved or auto-saved.
 0 initially.
 Each buffer has its own value of this variable.
@@ -2630,6 +2580,10 @@ inserts at the end of a line, and inserts when point is before a tab,
 until the tab is filled in.
 If `overwrite-mode-binary', self-insertion replaces newlines and tabs too.
 Automatically becomes buffer-local when set in any fashion.
+
+Normally, you shouldn't modify this variable by hand, but use the functions
+`overwrite-mode' and `binary-overwrite-mode' instead. However, you can
+customize the default value from the options menu.
 */ );
 
 #if 0 /* FSFmacs */
@@ -2780,64 +2734,89 @@ handled:
      DEFVAR_BUFFER_LOCAL() calls. */
 #define MARKED_SLOT(slot)					\
   if ((XINT (buffer_local_flags.slot) != -2 &&			\
-         XINT (buffer_local_flags.slot) != -3)			\
+       XINT (buffer_local_flags.slot) != -3)			\
       != !(NILP (XBUFFER (Vbuffer_local_symbols)->slot)))	\
   abort ()
 #include "bufslots.h"
 #undef MARKED_SLOT
 
-  Vprin1_to_string_buffer
-    = Fget_buffer_create (Fpurecopy (build_string (" prin1")));
-  /* Reset Vbuffer_alist again so that the above buf is magically
-     invisible */
-  Vbuffer_alist = Qnil;
-  /* Want no undo records for prin1_to_string_buffer */
-  Fbuffer_disable_undo (Vprin1_to_string_buffer);
-
   {
-    Lisp_Object scratch =
-      Fset_buffer (Fget_buffer_create (QSscratch));
+    Lisp_Object scratch = Fget_buffer_create (QSscratch);
+    Fset_buffer (scratch);
     /* Want no undo records for *scratch* until after Emacs is dumped */
     Fbuffer_disable_undo (scratch);
   }
 }
 
+/* Is PWD another name for `.' ? */
+static int
+directory_is_current_directory (char *pwd)
+{
+  Bufbyte *pwd_internal;
+  struct stat dotstat, pwdstat;
+
+  GET_C_CHARPTR_INT_FILENAME_DATA_ALLOCA (pwd, pwd_internal);
+
+  return (IS_DIRECTORY_SEP (*pwd_internal)
+	  && stat ((char *) pwd_internal, &pwdstat) == 0
+	  && stat (".", &dotstat) == 0
+	  && dotstat.st_ino == pwdstat.st_ino
+	  && dotstat.st_dev == pwdstat.st_dev
+	  && (int) strlen ((char *) pwd_internal) < MAXPATHLEN);
+}
+
+void
+init_initial_directory (void)
+{
+  /* This function can GC */
+
+  char *pwd;
+
+  initial_directory[0] = 0;
+
+  /* If PWD is accurate, use it instead of calling getcwd.  This is faster
+     when PWD is right, and may avoid a fatal error.  */
+  if ((pwd = getenv ("PWD")) != NULL
+      && directory_is_current_directory (pwd))
+    strcpy (initial_directory, pwd);
+  else if (getcwd (initial_directory, MAXPATHLEN) == NULL)
+    fatal ("`getcwd' failed: %s\n", strerror (errno));
+
+  /* Make sure pwd is DIRECTORY_SEP-terminated.
+     Maybe this should really use some standard subroutine
+     whose definition is filename syntax dependent.  */
+  {
+    int len = strlen (initial_directory);
+
+    if (! IS_DIRECTORY_SEP (initial_directory[len - 1]))
+      {
+	initial_directory[len] = DIRECTORY_SEP;
+	initial_directory[len + 1] = '\0';
+      }
+  }
+
+  /* XEmacs change: store buffer's default directory
+     using prefered (i.e. as defined at compile-time)
+     directory separator. --marcpa */
+#ifdef DOS_NT
+#define CORRECT_DIR_SEPS(s) \
+  do { if ('/' == DIRECTORY_SEP) dostounix_filename (s); \
+       else unixtodos_filename (s); \
+  } while (0)
+
+  CORRECT_DIR_SEPS(initial_directory);
+#endif
+}
+
 void
 init_buffer (void)
 {
   /* This function can GC */
-  char buf[MAXPATHLEN+1];
-  char *pwd;
-  struct stat dotstat, pwdstat;
-  int rc;
-
-  buf[0] = 0;
 
   Fset_buffer (Fget_buffer_create (QSscratch));
 
-  /* If PWD is accurate, use it instead of calling getcwd.  This is faster
-     when PWD is right, and may avoid a fatal error.  */
-  if ((pwd = getenv ("PWD")) != 0 && IS_DIRECTORY_SEP (*pwd)
-      && stat (pwd, &pwdstat) == 0
-      && stat (".", &dotstat) == 0
-      && dotstat.st_ino == pwdstat.st_ino
-      && dotstat.st_dev == pwdstat.st_dev
-      && (int) strlen (pwd) < MAXPATHLEN)
-    strcpy (buf, pwd);
-  else if (getcwd (buf, MAXPATHLEN) == NULL)
-    fatal ("`getcwd' failed: %s\n", strerror (errno));
-
-#ifndef VMS
-  /* Maybe this should really use some standard subroutine
-     whose definition is filename syntax dependent.  */
-  rc = strlen (buf);
-  if (!(IS_DIRECTORY_SEP (buf[rc - 1])))
-    {
-      buf[rc] = DIRECTORY_SEP;
-      buf[rc + 1] = '\0';
-    }
-#endif /* not VMS */
-  current_buffer->directory = build_string (buf);
+  current_buffer->directory =
+    build_ext_string (initial_directory, FORMAT_FILENAME);
 
 #if 0 /* FSFmacs */
   /* #### is this correct? */
