@@ -19,6 +19,7 @@ the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: Mule 2.0, FSF 19.30. */
+/* Partly sync'ed with 19.36.4 */
 
 #include <config.h>
 #include "lisp.h"
@@ -31,8 +32,8 @@ Boston, MA 02111-1307, USA.  */
 #include "process.h"
 #include "sysdep.h"
 #include "window.h"
-#ifdef MULE
-#include "mule-coding.h"
+#ifdef FILE_CODING
+#include "file-coding.h"
 #endif
 
 #include "systime.h"
@@ -43,25 +44,19 @@ Boston, MA 02111-1307, USA.  */
 
 #ifdef WINDOWSNT
 #define _P_NOWAIT 1	/* from process.h */
+#include <windows.h>
+#include "nt.h"
 #endif
 
 #ifdef DOS_NT
 /* When we are starting external processes we need to know whether they
    take binary input (no conversion) or text input (\n is converted to
-   \r\n).  Similar for output: if newlines are written as \r\n then it's
+   \r\n).  Similarly for output: if newlines are written as \r\n then it's
    text process output, otherwise it's binary.  */
 Lisp_Object Vbinary_process_input;
 Lisp_Object Vbinary_process_output;
 #endif /* DOS_NT */
 
-Lisp_Object Vexec_path, Vexec_directory, Vdata_directory, Vdoc_directory;
-Lisp_Object Vdata_directory_list;
-Lisp_Object Vconfigure_info_directory, Vsite_directory;
-Lisp_Object Vinfopath_internal;
-
-/* The default base directory XEmacs is installed under. */
-Lisp_Object Vprefix_directory;
-
 Lisp_Object Vshell_file_name;
 
 /* The environment to pass to all subprocesses when they are started.
@@ -87,7 +82,6 @@ int synch_process_retcode;
 /* Nonzero if this is termination due to exit.  */
 static int call_process_exited;
 
-#ifndef VMS  /* VMS version is in vmsproc.c.  */
 
 static Lisp_Object
 call_process_kill (Lisp_Object fdpid)
@@ -108,13 +102,6 @@ call_process_kill (Lisp_Object fdpid)
 static Lisp_Object
 call_process_cleanup (Lisp_Object fdpid)
 {
-#ifdef MSDOS
-  /* for MSDOS fdpid is really (fd . tempfile)  */
-  Lisp_Object file = Fcdr (fdpid);
-  close (XINT (Fcar (fdpid)));
-  if (strcmp (XSTRING_DATA (file), NULL_DEVICE) != 0)
-    unlink (XSTRING_DATA (file));
-#else /* not MSDOS */
   int fd = XINT (Fcar (fdpid));
   int pid = XINT (Fcdr (fdpid));
 
@@ -138,7 +125,6 @@ call_process_cleanup (Lisp_Object fdpid)
   }
   synch_process_alive = 0;
   close (fd);
-#endif /* not MSDOS */
   return Qnil;
 }
 
@@ -195,10 +181,6 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
   /* File to use for stderr in the child.
      t means use same as standard output.  */
   Lisp_Object error_file;
-#ifdef MSDOS
-  char *outf, *tempfile;
-  int outfilefd;
-#endif /* MSDOS */
 
   CHECK_STRING (args[0]);
 
@@ -231,7 +213,8 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
     current_dir = Funhandled_file_name_directory (current_dir);
     current_dir = expand_and_dir_to_file (current_dir, Qnil);
 #if 0
-  /* This breaks everything in the presence of ange-ftp-visited files */
+    /* This is in FSF, but it breaks everything in the presence of
+       ange-ftp-visited files, so away with it.  */
     if (NILP (Ffile_accessible_directory_p (current_dir)))
       report_file_error ("Setting current directory",
                          Fcons (current_buffer->directory, Qnil));
@@ -281,8 +264,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
 	    || EQ (buffer, Qt)
 	    || ZEROP (buffer)))
 	{
-	  Lisp_Object spec_buffer;
-	  spec_buffer = buffer;
+	  Lisp_Object spec_buffer = buffer;
 	  buffer = Fget_buffer (buffer);
 	  /* Mention the buffer name for a better error message.  */
 	  if (NILP (buffer))
@@ -312,43 +294,13 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
     report_file_error ("Searching for program", Fcons (args[0], Qnil));
   new_argv[0] = (char *) XSTRING_DATA (path);
 
-  filefd = open ((char *) XSTRING_DATA (infile), O_RDONLY, 0);
+  filefd = open ((char *) XSTRING_DATA (infile), O_RDONLY | OPEN_BINARY, 0);
   if (filefd < 0)
     report_file_error ("Opening process input file", Fcons (infile, Qnil));
 
-#ifdef MSDOS
-  /* These vars record information from process termination.
-     Clear them now before process can possibly terminate,
-     to avoid timing error if process terminates soon.  */
-  synch_process_death = 0;
-  synch_process_retcode = 0;
-
-  if ((outf = egetenv ("TMP")) || (outf = egetenv ("TEMP")))
-    strcpy (tempfile = alloca (strlen (outf) + 20), outf);
-  else
-    {
-      tempfile = alloca (20);
-      *tempfile = '\0';
-    }
-  dostounix_filename (tempfile);
-  if (*tempfile == '\0' || tempfile[strlen (tempfile) - 1] != '/')
-    strcat (tempfile, "/");
-  strcat (tempfile, "detmp.XXX");
-  mktemp (tempfile);
-
-  outfilefd = creat (tempfile, S_IREAD | S_IWRITE);
-  if (outfilefd < 0)
-    {
-      close (filefd);
-      report_file_error ("Opening process output file",
-			 Fcons (tempfile, Qnil));
-    }
-#endif /* MSDOS */
-
-#ifndef MSDOS
   if (INTP (buffer))
     {
-      fd[1] = open (NULL_DEVICE, O_WRONLY, 0);
+      fd[1] = open (NULL_DEVICE, O_WRONLY | OPEN_BINARY, 0);
       fd[0] = -1;
     }
   else
@@ -359,40 +311,9 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
       set_exclusive_use (fd[0]);
 #endif
     }
-#else /* MSDOS */
+
   {
-    char *outf;
-
-  if (INTP (buffer))
-    outf = NULL_DEVICE;
-  else
-    {
-	/* DOS can't create pipe for interprocess communication,
-	   so redirect child process's standard output to temporary file
-	   and later read the file. */
-
-      if ((outf = egetenv ("TMP")) || (outf = egetenv ("TEMP")))
-	{
-	  strcpy (tempfile, outf);
-	  dostounix_filename (tempfile);
-	}
-      else
-        *tempfile = '\0';
-      if (strlen (tempfile) == 0 || tempfile[strlen (tempfile) - 1] != '/')
-	strcat (tempfile, "/");
-      strcat (tempfile, "demacs.XXX");
-      mktemp (tempfile);
-      outf = tempfile;
-    }
-
-    if ((fd[1] = creat (outf, S_IREAD | S_IWRITE)) < 0)
-      report_file_error ("Can't open temporary file", Qnil);
-    fd[0] = -1;
-    }
-#endif /* MSDOS */
-
-   {
-     /* child_setup must clobber environ in systems with true vfork.
+    /* child_setup must clobber environ in systems with true vfork.
 	Protect it from permanent change.  */
      REGISTER char **save_environ = environ;
      REGISTER int fd1 = fd[1];
@@ -418,33 +339,19 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
     synch_process_death = 0;
     synch_process_retcode = 0;
 
-#ifdef MSDOS
-    /* ??? Someone who knows MSDOG needs to check whether this properly
-       closes all descriptors that it opens.  */
-    pid = run_msdos_command (new_argv, current_dir, filefd, outfilefd);
-    close (outfilefd);
-    fd1 = -1; /* No harm in closing that one!  */
-    fd[0] = open (tempfile, NILP (Vbinary_process_output) ? O_TEXT :
-			O_BINARY);
-    if (fd[0] < 0)
-      {
-	unlink (tempfile);
-	close (filefd);
-	report_file_error ("Cannot re-open temporary file", Qnil);
-      }
-#else /* not MSDOS */
     if (NILP (error_file))
-      fd_error = open (NULL_DEVICE, O_WRONLY);
+      fd_error = open (NULL_DEVICE, O_WRONLY | OPEN_BINARY);
     else if (STRINGP (error_file))
       {
+	fd_error = open ((CONST char *) XSTRING_DATA (error_file),
 #ifdef DOS_NT
-	fd_error = open (XSTRING_DATA (error_file),
 			 O_WRONLY | O_TRUNC | O_CREAT | O_TEXT,
-			 S_IREAD | S_IWRITE);
+			 S_IREAD | S_IWRITE
 #else  /* not DOS_NT */
-	fd_error =
-	  creat ((CONST char *) XSTRING_DATA (error_file), 0666);
+			 O_WRONLY | O_TRUNC | O_CREAT | OPEN_BINARY,
+			 CREAT_MODE
 #endif /* not DOS_NT */
+			 );
       }
 
     if (fd_error < 0)
@@ -485,7 +392,6 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
     if (fd_error >= 0)
       close (fd_error);
 
-#endif /* not MSDOS */
 #endif /* not WINDOWSNT */
 
     environ = save_environ;
@@ -530,28 +436,21 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
     /* Enable sending signal if user quits below.  */
     call_process_exited = 0;
 
-#ifdef MSDOS
-    /* MSDOS needs different cleanup information.  */
-    record_unwind_protect (call_process_cleanup,
-                           Fcons (make_int (fd[0]),
-                                  build_string (tempfile)));
-#else /* not MSDOS */
     record_unwind_protect (call_process_cleanup,
                            Fcons (make_int (fd[0]), make_int (pid)));
-#endif /* not MSDOS */
 
     /* FSFmacs calls Fset_buffer() here.  We don't have to because
        we can insert into buffers other than the current one. */
     if (EQ (buffer, Qt))
       XSETBUFFER (buffer, current_buffer);
     instream = make_filedesc_input_stream (fd[0], 0, -1, LSTR_ALLOW_QUIT);
-#ifdef MULE
+#ifdef FILE_CODING
     instream =
       make_decoding_input_stream
 	(XLSTREAM (instream),
 	 Fget_coding_system (Vcoding_system_for_read));
     Lstream_set_character_mode (XLSTREAM (instream));
-#endif /* MULE */
+#endif
     NGCPRO1 (instream);
     while (1)
       {
@@ -582,6 +481,18 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
 	if (nread == 0)
 	  break;
 
+#ifdef DOS_NT
+       /* Until we pull out of MULE things like
+	  make_decoding_input_stream(), we do the following which is
+	  less elegant. --marcpa */
+       {
+	 int lf_count = 0;
+	 if (NILP (Vbinary_process_output)) {
+	   nread = crlf_to_lf(nread, bufptr, &lf_count);
+         }
+       }
+#endif
+
 	total_read += nread;
 
 	if (!NILP (buffer))
@@ -607,10 +518,8 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
     NUNGCPRO;
 
     QUIT;
-#ifndef MSDOS
     /* Wait for it to terminate, unless it already has.  */
     wait_for_termination (pid);
-#endif
 
     /* Don't kill any children that the subprocess may have left behind
        when exiting.  */
@@ -623,9 +532,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you
   }
 }
 
-#endif /* VMS */
 
-#ifndef VMS /* VMS version is in vmsproc.c.  */
 
 /* This is the last thing run in a newly forked inferior
    either synchronous or asynchronous.
@@ -657,10 +564,6 @@ void
 child_setup (int in, int out, int err, char **new_argv,
 	     CONST char *current_dir)
 {
-#ifdef MSDOS
-  /* The MSDOS port of gcc cannot fork, vfork, ... so we must call system
-     instead.  */
-#else /* not MSDOS */
   char **env;
   char *pwd;
 #ifdef WINDOWSNT
@@ -673,7 +576,7 @@ child_setup (int in, int out, int err, char **new_argv,
     nice (- emacs_priority);
 #endif
 
-#if !defined (NO_SUBPROCESSES)
+#if !defined (NO_SUBPROCESSES) && !defined (WINDOWSNT)
   /* Close Emacs's descriptors that this process should not have.  */
   close_process_descs ();
 #endif /* not NO_SUBPROCESSES */
@@ -720,9 +623,8 @@ child_setup (int in, int out, int err, char **new_argv,
   {
     REGISTER Lisp_Object tem;
     REGISTER char **new_env;
-    REGISTER int new_length;
+    REGISTER int new_length = 0;
 
-    new_length = 0;
     for (tem = Vprocess_environment;
 	 (CONSP (tem)
 	  && STRINGP (XCAR (tem)));
@@ -777,6 +679,7 @@ child_setup (int in, int out, int err, char **new_argv,
   }
 #ifdef WINDOWSNT
   prepare_standard_handles (in, out, err, handles);
+  set_process_dir (current_dir);
 #else  /* not WINDOWSNT */
   /* Make sure that in, out, and err are not actually already in
      descriptors zero, one, or two; this could happen if Emacs is
@@ -841,10 +744,9 @@ child_setup (int in, int out, int err, char **new_argv,
   environ = env;
   execvp (new_argv[0], new_argv);
 
-  stdout_out ("Cant't exec program %s\n", new_argv[0]);
+  stdout_out ("Can't exec program %s\n", new_argv[0]);
   _exit (1);
 #endif /* not WINDOWSNT */
-#endif /* not MSDOS */
 }
 
 /* Move the file descriptor FD so that its number is not less than MIN.
@@ -925,6 +827,8 @@ When invoked interactively, prints the value in the echo area.
       if (NILP (v))
 	message ("%s not defined in environment", XSTRING_DATA (var));
       else
+	/* #### Should use Fprin1_to_string or Fprin1 to handle string
+           containing quotes correctly.  */
 	message ("\"%s\"", value);
     }
   RETURN_UNGCPRO (v);
@@ -943,7 +847,6 @@ egetenv (CONST char *var)
   else
     return 0;
 }
-#endif /* not VMS */
 
 
 void
@@ -951,7 +854,6 @@ init_callproc (void)
 {
   /* This function can GC */
   REGISTER char *sh;
-  Lisp_Object tempdir;
 
   Vprocess_environment = Qnil;
   /* jwz: always initialize Vprocess_environment, so that egetenv() works
@@ -959,170 +861,37 @@ init_callproc (void)
   {
     char **envp;
     for (envp = environ; envp && *envp; envp++)
-      Vprocess_environment = Fcons (build_ext_string (*envp, FORMAT_OS),
-				    Vprocess_environment);
+      {
+	Vprocess_environment = Fcons (build_ext_string (*envp, FORMAT_OS),
+				      Vprocess_environment);
+      }
   }
 
-  /* jwz: don't do these things when in temacs (this used to be the case by
-     virtue of egetenv() always returning 0, but that has been changed).
-   */
-#ifndef CANNOT_DUMP
-  if (!initialized)
-    {
-      Vdata_directory = Qnil;
-      Vsite_directory = Qnil;
-      Vdoc_directory  = Qnil;
-      Vexec_path      = Qnil;
-    }
-  else
-#endif
-    {
-      char *data_dir = egetenv ("EMACSDATA");
-      char *site_dir = egetenv ("EMACSSITE");
-      char *doc_dir  = egetenv ("EMACSDOC");
-
-#ifdef PATH_DATA
-      if (!data_dir)
-	data_dir = (char *) PATH_DATA;
-#endif
-#ifdef PATH_DOC
-      if (!doc_dir)
-	doc_dir = (char *) PATH_DOC;
-#endif
-#ifdef PATH_SITE
-      if (!site_dir)
-	site_dir = (char *) PATH_SITE;
-#endif
-
-      if (data_dir)
-	Vdata_directory = Ffile_name_as_directory
-	  (build_string (data_dir));
-      else
-	Vdata_directory = Qnil;
-      if (doc_dir)
-	Vdoc_directory = Ffile_name_as_directory
-	  (build_string (doc_dir));
-      else
-	Vdoc_directory = Qnil;
-      if (site_dir)
-	Vsite_directory = Ffile_name_as_directory
-	  (build_string (site_dir));
-      else
-	Vsite_directory = Qnil;
-
-      /* Check the EMACSPATH environment variable, defaulting to the
-	 PATH_EXEC path from paths.h.  */
-      Vexec_path = decode_env_path ("EMACSPATH",
-#ifdef PATH_EXEC
-				    PATH_EXEC
-#else
-				    0
-#endif
-				    );
-    }
-
-  if (NILP (Vexec_path))
-    Vexec_directory = Qnil;
-      else
-    Vexec_directory = Ffile_name_as_directory
-      (Fcar (Vexec_path));
-
-  if (initialized)
-    Vexec_path = nconc2 (decode_env_path ("PATH", 0),
-                         Vexec_path);
-
-  if (!NILP (Vexec_directory))
-    {
-      tempdir = Fdirectory_file_name (Vexec_directory);
-      if (access ((char *) XSTRING_DATA (tempdir), 0) < 0)
-	{
-	  /* If the hard-coded path is bogus, fail silently.
-	     This will allow the normal heuristics to make an attempt. */
-#if 0
-	  warn_when_safe
-	    (Qpath, Qwarning,
-	     "Warning: machine-dependent data dir (%s) does not exist.\n",
-	     XSTRING_DATA (Vexec_directory));
-#else
-	  Vexec_directory = Qnil;
-#endif
-	}
-    }
-
-  if (!NILP (Vdata_directory))
-    {
-      tempdir = Fdirectory_file_name (Vdata_directory);
-      if (access ((char *) XSTRING_DATA (tempdir), 0) < 0)
-	{
-	  /* If the hard-coded path is bogus, fail silently.
-	     This will allow the normal heuristics to make an attempt. */
-#if 0
-	  warn_when_safe
-	    (Qpath, Qwarning,
-	     "Warning: machine-independent data dir (%s) does not exist.\n",
-	     XSTRING_DATA (Vdata_directory));
-#else
-	  Vdata_directory = Qnil;
-#endif
-	}
-    }
-
-  if (!NILP (Vsite_directory))
-    {
-      tempdir = Fdirectory_file_name (Vsite_directory);
-      if (access ((char *) XSTRING_DATA (tempdir), 0) < 0)
-	{
-	  /* If the hard-coded path is bogus, fail silently.
-	     This will allow the normal heuristics to make an attempt. */
-#if 0
-	  warn_when_safe
-	    (Qpath, Qwarning,
-	     "Warning: machine-independent site dir (%s) does not exist.\n",
-	     XSTRING_DATA (Vsite_directory));
-#else
-	  Vsite_directory = Qnil;
-#endif
-	}
-    }
-
-#ifdef PATH_PREFIX
-  Vprefix_directory = build_string ((char *) PATH_PREFIX);
-#else
-  Vprefix_directory = Qnil;
-#endif
-
-#ifdef VMS
-  Vshell_file_name = build_string ("*dcl*");
-#elif defined(WINDOWSNT)
+#ifdef WINDOWSNT
+  /* Sync with FSF Emacs 19.34.6 note: this is not in 19.34.6. --marcpa */
   /*
   ** If NT then we look at COMSPEC for the shell program.
   */
   sh = egetenv ("COMSPEC");
-  {
-    char *tem;
-	/*
-	** If COMSPEC has been set, then convert the
-	** DOS formatted name into a UNIX format. Then
-	** create a LISP object.
-	*/
-    if (sh)
-    {
-		tem = (char *) alloca (strlen (sh) + 1);
-		dostounix_filename (strcpy (tem, sh));
-		Vshell_file_name = build_string (tem);
-	}
-	/*
-	** Odd, no COMSPEC, so let's default to our
-	** best guess for NT.
-	*/
-	else
-	{
-		Vshell_file_name = build_string ("/WINNT/system32/cmd.exe");
-	}
-  }
-#else /* not VMS or WINDOWSNT */
+  /*
+  ** If COMSPEC has been set, then convert the
+  ** DOS formatted name into a UNIX format. Then
+  ** create a LISP object.
+  */
+  if (sh)
+    Vshell_file_name = build_string (sh);
+  /*
+  ** Odd, no COMSPEC, so let's default to our
+  ** best guess for NT.
+  */
+  else
+    Vshell_file_name = build_string ("\\WINNT\\system32\\cmd.exe");
+
+#else /* not WINDOWSNT */
+
   sh = (char *) egetenv ("SHELL");
   Vshell_file_name = build_string (sh ? sh : "/bin/sh");
+
 #endif
 }
 
@@ -1145,10 +914,8 @@ set_process_environment (void)
 void
 syms_of_callproc (void)
 {
-#ifndef VMS
   DEFSUBR (Fcall_process_internal);
   DEFSUBR (Fgetenv);
-#endif
 }
 
 void
@@ -1170,46 +937,6 @@ vars_of_callproc (void)
   DEFVAR_LISP ("shell-file-name", &Vshell_file_name /*
 *File name to load inferior shells from.
 Initialized from the SHELL environment variable.
-*/ );
-
-  DEFVAR_LISP ("exec-path", &Vexec_path /*
-*List of directories to search programs to run in subprocesses.
-Each element is a string (directory name) or nil (try default directory).
-*/ );
-
-  DEFVAR_LISP ("exec-directory", &Vexec_directory /*
-Directory of architecture-dependent files that come with XEmacs,
-especially executable programs intended for XEmacs to invoke.
-*/ );
-
-  DEFVAR_LISP ("data-directory", &Vdata_directory /*
-Directory of architecture-independent files that come with XEmacs,
-intended for XEmacs to use.
-Use of this variable in new code is almost never correct.  See the
-function `locate-data-directory' and the variable `data-directory-list'.
-*/ );
-
-  DEFVAR_LISP ("data-directory-list", &Vdata_directory_list /*
-List of directories of architecture-independent files that come with XEmacs
-or were installed as packages, and are intended for XEmacs to use.
-*/ );
-  Vdata_directory_list = Qnil;
-
-  DEFVAR_LISP ("site-directory", &Vsite_directory /*
-Directory of architecture-independent files that do not come with XEmacs,
-intended for XEmacs to use.
-*/ );
-
-  /* FSF puts the DOC file into data-directory.  They do a bunch of
-     contortions to attempt to put everything into the DOC file
-     whether the support is there or not. */
-  DEFVAR_LISP ("doc-directory", &Vdoc_directory /*
-Directory containing the DOC file that comes with XEmacs.
-This is usually the same as exec-directory.
-*/ );
-
-  DEFVAR_LISP ("prefix-directory", &Vprefix_directory /*
-The default directory under which XEmacs is installed.
 */ );
 
   DEFVAR_LISP ("process-environment", &Vprocess_environment /*
@@ -1219,32 +946,3 @@ The environment which Emacs inherits is placed in this variable
 when Emacs starts.
 */ );
 }
-
-void
-complex_vars_of_callproc (void)
-{
-  DEFVAR_LISP ("configure-info-directory", &Vconfigure_info_directory /*
-For internal use by the build procedure only.
-This is the name of the directory in which the build procedure installed
-Emacs's info files; the default value for Info-default-directory-list
-includes this.
-*/ );
-
-#ifdef PATH_INFO
-  Vconfigure_info_directory =
-    Ffile_name_as_directory (build_string (PATH_INFO));
-#else
-  Vconfigure_info_directory = Qnil;
-#endif
-
-  DEFVAR_LISP ("infopath-internal", &Vinfopath_internal /*
-The configured initial value of Info-default-directory-list.
-*/ );
-
-#ifdef PATH_INFOPATH
-  Vinfopath_internal = build_string (PATH_INFOPATH);
-#else
-  Vinfopath_internal =
-    build_string ("/usr/info:/usr/local/info:/usr/lib/texmf/doc/info:/usr/local/lib/texmf/doc/info");
-#endif
-}
