@@ -54,6 +54,9 @@ Boston, MA 02111-1307, USA.  */
 #endif
 
 Lisp_Object Vdefault_x_device;
+#if defined(MULE) && (defined(LWLIB_MENUBARS_MOTIF) || defined(HAVE_XIM) || defined (USE_XFONTSET))
+Lisp_Object Vx_app_defaults_directory;
+#endif
 
 /* Qdisplay in general.c */
 Lisp_Object Qx_error;
@@ -77,6 +80,9 @@ static XrmOptionDescRec emacs_options[] =
   {"-scrollbar-width",       "*EmacsFrame.scrollBarWidth",      XrmoptionSepArg, NULL},
   {"-scrollbar-height",      "*EmacsFrame.scrollBarHeight",     XrmoptionSepArg, NULL},
 
+  {"-privatecolormap", ".privateColormap", XrmoptionNoArg,  "yes"},
+  {"-visual",   ".EmacsVisual",	    XrmoptionSepArg, NULL},
+
   /* #### Beware!  If the type of the shell changes, update this. */
   {"-T",        "*TopLevelEmacsShell.title",    XrmoptionSepArg, NULL},
   {"-wn",       "*TopLevelEmacsShell.title",    XrmoptionSepArg, NULL},
@@ -89,18 +95,16 @@ static XrmOptionDescRec emacs_options[] =
   {"-fontset",  "*FontSet",                     XrmoptionSepArg, NULL},
 };
 
-static void validify_resource_string (char *str);
-
 /* Functions to synchronize mirroring resources and specifiers */
 int in_resource_setting;
-int in_specifier_change_function;
-
 
 /************************************************************************/
 /*                          helper functions                            */
 /************************************************************************/
 
-static struct device *
+/* JH 97/11/25 removed the static declaration because I need it during setup in event-Xt... */
+struct device * get_device_from_display_1 (Display *dpy);
+struct device *
 get_device_from_display_1 (Display *dpy)
 {
   Lisp_Object devcons, concons;
@@ -120,16 +124,24 @@ get_device_from_display (Display *dpy)
 {
   struct device *d = get_device_from_display_1 (dpy);
 
+#if !defined(INFODOCK)
+# define FALLBACK_RESOURCE_NAME "xemacs"
+# else
+# define FALLBACK_RESOURCE_NAME "infodock"
+#endif
+
   if (!d) {
     /* This isn't one of our displays.  Let's crash? */
     stderr_out
       ("\n%s: Fatal X Condition.  Asked about display we don't own: \"%s\"\n",
        (STRINGP (Vinvocation_name) ?
-	(char *) XSTRING_DATA (Vinvocation_name) : "xemacs"),
+	(char *) XSTRING_DATA (Vinvocation_name) : FALLBACK_RESOURCE_NAME),
        DisplayString (dpy) ? DisplayString (dpy) : "???");
     abort();
   }
 
+#undef FALLBACK_RESOURCE_NAME
+
   return d;
 }
 
@@ -141,7 +153,7 @@ decode_x_device (Lisp_Object device)
   return XDEVICE (device);
 }
 
-Display *
+static Display *
 get_x_display (Lisp_Object device)
 {
   return DEVICE_X_DISPLAY (decode_x_device (device));
@@ -194,7 +206,7 @@ sanity_check_geometry_resource (Display *dpy)
 "You should always use \".geometry\" or \"*EmacsFrame.geometry\" instead.\n",
 		  app_name, (char *) value.addr,
 		  app_class, (char *) value.addr);
-      suppress_early_backtrace = 1;
+      suppress_early_error_handler_backtrace = 1;
       error ("Invalid geometry resource");
     }
 }
@@ -202,10 +214,9 @@ sanity_check_geometry_resource (Display *dpy)
 static void
 x_init_device_class (struct device *d)
 {
-  Display *dpy = DEVICE_X_DISPLAY (d);
-  if (DisplayCells (dpy, DefaultScreen (dpy)) > 2)
+  if (DEVICE_X_DEPTH(d) > 2)
     {
-      switch (DefaultVisualOfScreen (DefaultScreenOfDisplay (dpy))->class)
+      switch (DEVICE_X_VISUAL(d)->class)
 	{
 	case StaticGray:
 	case GrayScale:
@@ -219,16 +230,113 @@ x_init_device_class (struct device *d)
     DEVICE_CLASS (d) = Qmono;
 }
 
+/*
+ * Figure out what application name to use for xemacs
+ *
+ * Since we have decomposed XtOpenDisplay into XOpenDisplay and
+ * XtDisplayInitialize, we no longer get this for free.
+ *
+ * If there is a `-name' argument in argv, use that.
+ * Otherwise use the last component of argv[0].
+ *
+ * I have removed the gratuitous use of getenv("RESOURCE_NAME")
+ * which was in X11R5, but left the matching of any prefix of `-name'.
+ * Finally, if all else fails, return `xemacs', as it is more
+ * appropriate (X11R5 returns `main').
+ */
+static char *
+compute_x_app_name (int argc, char **argv)
+{
+  int i;
+  char *ptr;
+
+  for (i = 1; i < argc - 1; i++)
+    if (!strncmp(argv[i], "-name", max (2, strlen (argv[1]))))
+      return argv[i+1];
+
+  if (argc > 0 && argv[0] && *argv[0])
+    return (ptr = strrchr (argv[0], '/')) ? ++ptr : argv[0];
+
+  return "xemacs";
+}
+
+/*
+ * This function figures out whether the user has any resources of the
+ * form "XEmacs.foo" or "XEmacs*foo".
+ *
+ * Currently we only consult the display's global resources; to look
+ * for screen specific resources, we would need to also consult:
+ * xdefs = XScreenResourceString(ScreenOfDisplay(dpy, scrno));
+ */
+static int
+have_xemacs_resources_in_xrdb (Display *dpy)
+{
+  char *xdefs, *key;
+  int len;
+
+#ifdef INFODOCK
+  key = "InfoDock";
+#else
+  key = "XEmacs";
+#endif
+  len = strlen (key);
+
+  if (!dpy)
+    return 0;
+
+  xdefs = XResourceManagerString (dpy);      /* don't free - owned by X */
+  while (xdefs && *xdefs)
+    {
+      if (strncmp (xdefs, key, len) == 0  &&
+          (xdefs[len] == '*' || xdefs[len] == '.'))
+        return 1;
+
+      while (*xdefs && *xdefs++ != '\n')     /* find start of next entry.. */
+        ;
+    }
+
+  return 0;
+}
+
+/* Only the characters [-_A-Za-z0-9] are allowed in the individual
+   components of a resource.  Convert invalid characters to `-' */
+
+static char valid_resource_char_p[256];
+
+static void
+validify_resource_component (char *str, size_t len)
+{
+  for (; len; len--, str++)
+    if (!valid_resource_char_p[(unsigned char) (*str)])
+      *str = '-';
+}
+
+static void
+Dynarr_add_validified_lisp_string (char_dynarr *cda, Lisp_Object str)
+{
+  Bytecount len = XSTRING_LENGTH (str);
+  Dynarr_add_many (cda, (char *) XSTRING_DATA (str), len);
+  validify_resource_component (Dynarr_atp (cda, Dynarr_length (cda) - len), len);
+}
+
 static void
 x_init_device (struct device *d, Lisp_Object props)
 {
   Lisp_Object display;
   Lisp_Object device;
   Display *dpy;
+  Widget app_shell;
   int argc;
   char **argv;
   CONST char *app_class;
+  CONST char *app_name;
   CONST char *disp_name;
+  Arg xargs[6];
+  Cardinal numargs;
+  Visual *visual = NULL;
+  int depth = 8;		/* shut up the compiler */
+  Colormap cmap;
+  int screen;
 
   XSETDEVICE (device, d);
   display = DEVICE_CONNECTION (d);
@@ -237,27 +345,52 @@ x_init_device (struct device *d, Lisp_Object props)
 
   make_argc_argv (Vx_initial_argv_list, &argc, &argv);
 
-  if (STRINGP (Vx_emacs_application_class) &&
-      XSTRING_LENGTH (Vx_emacs_application_class) > 0)
-    GET_C_STRING_CTEXT_DATA_ALLOCA (Vx_emacs_application_class, app_class);
-  else
-    app_class = "Emacs";
-
   GET_C_STRING_CTEXT_DATA_ALLOCA (display, disp_name);
 
+  /*
+   * Break apart the old XtOpenDisplay call into XOpenDisplay and
+   * XtDisplayInitialize so we can figure out whether there
+   * are any XEmacs resources in the resource database before
+   * we initialize Xt.  This is so we can automagically support
+   * both `Emacs' and `XEmacs' application classes.
+   */
   slow_down_interrupts ();
-  /* The Xt code can't deal with signals here.  Yuck. */
-  dpy = DEVICE_X_DISPLAY (d) =
-    XtOpenDisplay (Xt_app_con, disp_name, NULL, app_class, emacs_options,
-                   XtNumber (emacs_options), &argc, argv);
+  /* May not be needed but XtOpenDisplay could not deal with signals here. */
+  dpy = DEVICE_X_DISPLAY (d) = XOpenDisplay (disp_name);
   speed_up_interrupts ();
 
   if (dpy == 0)
     {
-      suppress_early_backtrace = 1;
+      suppress_early_error_handler_backtrace = 1;
       signal_simple_error ("X server not responding\n", display);
     }
 
+  if (STRINGP (Vx_emacs_application_class) &&
+      XSTRING_LENGTH (Vx_emacs_application_class) > 0)
+    GET_C_STRING_CTEXT_DATA_ALLOCA (Vx_emacs_application_class, app_class);
+  else
+    {
+      app_class = (NILP (Vx_emacs_application_class)  &&
+                   have_xemacs_resources_in_xrdb (dpy))
+#ifdef INFODOCK
+                  ? "InfoDock"
+#else
+                  ? "XEmacs"
+#endif
+                  : "Emacs";
+      /* need to update Vx_emacs_application_class: */
+      Vx_emacs_application_class = build_string (app_class);
+    }
+
+  slow_down_interrupts ();
+  /* May not be needed but XtOpenDisplay could not deal with signals here.
+     Yuck. */
+  XtDisplayInitialize (Xt_app_con, dpy, compute_x_app_name (argc, argv),
+                       app_class, emacs_options,
+                       XtNumber (emacs_options), &argc, argv);
+  speed_up_interrupts ();
+
+  screen = DefaultScreen(dpy);
   if (NILP (Vdefault_x_device))
     Vdefault_x_device = device;
 
@@ -265,22 +398,32 @@ x_init_device (struct device *d, Lisp_Object props)
 #if defined(LWLIB_MENUBARS_MOTIF) || defined(HAVE_XIM) || defined (USE_XFONTSET)
   {
     /* Read in locale-specific resources from
-       data-directory/app-defaults/$LANG/emacs-application-class.
+       data-directory/app-defaults/$LANG/Emacs.
        This is in addition to the standard app-defaults files, and
        does not override resources defined elsewhere */
     CONST char *data_dir;
-    char path[MAXPATHLEN];
+    char *path;
     XrmDatabase db = XtDatabase (dpy); /* ### XtScreenDatabase(dpy) ? */
     CONST char *locale = XrmLocaleOfDatabase (db);
 
-    if (STRINGP (Vdata_directory) && XSTRING_LENGTH (Vdata_directory) > 0)
+    if (STRINGP (Vx_app_defaults_directory) &&
+	XSTRING_LENGTH (Vx_app_defaults_directory) > 0)
       {
-	GET_C_STRING_FILENAME_DATA_ALLOCA (Vdata_directory, data_dir);
-	sprintf (path, "%sapp-defaults/%s/%s", data_dir, locale, app_class);
+	GET_C_STRING_FILENAME_DATA_ALLOCA(Vx_app_defaults_directory, data_dir);
+	path = (char *)alloca (strlen (data_dir) + strlen (locale) + 7);
+	sprintf (path, "%s%s/Emacs", data_dir, locale);
 	if (!access (path, R_OK))
 	  XrmCombineFileDatabase (path, &db, False);
       }
-  }
+    else if (STRINGP (Vdata_directory) && XSTRING_LENGTH (Vdata_directory) > 0)
+      {
+	GET_C_STRING_FILENAME_DATA_ALLOCA (Vdata_directory, data_dir);
+	path = (char *)alloca (strlen (data_dir) + 13 + strlen (locale) + 7);
+	sprintf (path, "%sapp-defaults/%s/Emacs", data_dir, locale);
+	if (!access (path, R_OK))
+	  XrmCombineFileDatabase (path, &db, False);
+      }
+ }
 #endif /* LWLIB_MENUBARS_MOTIF or HAVE_XIM USE_XFONTSET */
 #endif /* MULE */
 
@@ -291,39 +434,132 @@ x_init_device (struct device *d, Lisp_Object props)
   DEVICE_NAME (d) = Fcopy_sequence (DEVICE_NAME (d));
   /* colons and periods can't appear in individual elements of resource
      strings */
-  validify_resource_string ((char *) XSTRING_DATA (DEVICE_NAME (d)));
-  DEVICE_XT_APP_SHELL (d) = XtAppCreateShell (NULL, app_class,
-					      applicationShellWidgetClass,
-					      dpy, NULL, 0);
 
+  XtGetApplicationNameAndClass (dpy, (char **) &app_name, (char **) &app_class);
+  /* search for a matching visual if requested by the user, or setup the display default */
+  numargs = 0;
+  {
+    char *buf1 = (char *)alloca (strlen (app_name) + 17);
+    char *buf2 = (char *)alloca (strlen (app_class) + 17);
+    char *type;
+    XrmValue value;
+
+    sprintf (buf1, "%s.emacsVisual", app_name);
+    sprintf (buf2, "%s.EmacsVisual", app_class);
+    if (XrmGetResource (XtDatabase (dpy), buf1, buf2, &type, &value) == True)
+      {
+	int cnt = 0, vis_class= PseudoColor;
+	XVisualInfo vinfo;
+	char *res, *str = (char*)value.addr;
+
+	if      (strncmp(str, "StaticGray", 10) == 0)	cnt = 10, vis_class = StaticGray;
+	else if (strncmp(str, "StaticColor", 11) == 0)	cnt = 11, vis_class = StaticColor;
+	else if (strncmp(str, "TrueColor", 9) == 0)	cnt = 9,  vis_class = TrueColor;
+	else if (strncmp(str, "GrayScale", 9) == 0)	cnt = 9,  vis_class = GrayScale;
+	else if (strncmp(str, "PseudoColor", 11) == 0)	cnt = 11, vis_class = PseudoColor;
+	else if (strncmp(str, "DirectColor", 11) == 0)	cnt = 11, vis_class = DirectColor;
+	if (cnt)
+	  {
+	    res = str + cnt;
+	    depth = atoi(res);
+	    if (depth == 0)
+	      {
+		stderr_out("Invalid Depth specification in %s... ignoring...\n",(char*)str);
+	      }
+	    else
+	      {
+		if (XMatchVisualInfo(dpy, screen, depth, vis_class, &vinfo))
+		  {
+		    visual = vinfo.visual;
+		  }
+		else
+		  {
+		    stderr_out("Can't match the requested visual %s... using defaults\n",str);
+		  }
+	      }
+	  }
+	else
+	  {
+	    stderr_out("Invalid Visual specification in %s... ignoring.\n",(char*)str);
+	  }
+      }
+    if (visual == NULL)
+      {
+	visual = DefaultVisual(dpy, screen);
+	depth = DefaultDepth(dpy, screen);
+      }
+
+    /* If we've got the same visual as the default and it's PseudoColor,
+       check to see if the user specified that we need a private colormap */
+    if (visual == DefaultVisual(dpy, screen))
+      {
+	sprintf (buf1, "%s.privateColormap", app_name);
+	sprintf (buf2, "%s.PrivateColormap", app_class);
+	if ((visual->class == PseudoColor) &&
+	    (XrmGetResource (XtDatabase (dpy), buf1, buf2, &type, &value) == True))
+	  {
+	     cmap = XCopyColormapAndFree(dpy, DefaultColormap(dpy, screen));
+	  }
+	else
+	  {
+	    cmap = DefaultColormap(dpy, screen);
+	  }
+      }
+    else
+      {
+	/* We have to create a matching colormap anyway...
+	   ### think about using standard colormaps (need the Xmu libs?) */
+	cmap = XCreateColormap(dpy, RootWindow(dpy, screen), visual, AllocNone);
+	XInstallColormap(dpy, cmap);
+      }
+  }
+  XtSetArg(xargs[numargs],XtNvisual, visual); numargs++;
+  XtSetArg(xargs[numargs],XtNdepth, depth); numargs++;
+  XtSetArg(xargs[numargs],XtNcolormap, cmap); numargs++;
+  DEVICE_X_VISUAL (d) = visual;
+  DEVICE_X_COLORMAP (d) = cmap;
+  DEVICE_X_DEPTH (d) = depth;
+
+  validify_resource_component ((char *) XSTRING_DATA (DEVICE_NAME (d)),
+			       XSTRING_LENGTH (DEVICE_NAME (d)));
+  app_shell = XtAppCreateShell (NULL, app_class,
+				applicationShellWidgetClass,
+				dpy, xargs, numargs);
+
+  DEVICE_XT_APP_SHELL (d) = app_shell;
 #ifdef HAVE_XIM
   XIM_init_device(d);
 #endif /* HAVE_XIM */
 
+  /* Realize the app_shell so that its window exists for GC creation purposes,
+     and set it to the size of the root window for child placement purposes */
+  {
+    Screen *scrn = ScreenOfDisplay(dpy, screen);
+    int screen_width, screen_height;
+    screen_width = WidthOfScreen(scrn);
+    screen_height = HeightOfScreen(scrn);
+    numargs = 0;
+    XtSetArg (xargs[numargs], XtNmappedWhenManaged, False); numargs++;
+    XtSetArg (xargs[numargs], XtNx, 0); numargs++;
+    XtSetArg (xargs[numargs], XtNy, 0); numargs++;
+    XtSetArg (xargs[numargs], XtNwidth,  screen_width); numargs++;
+    XtSetArg (xargs[numargs], XtNheight, screen_height); numargs++;
+    XtSetValues (app_shell, xargs, numargs);
+    XtRealizeWidget (app_shell);
+  }
 #ifdef HAVE_SESSION
   {
-    Arg al[3];
-    Widget shell = DEVICE_XT_APP_SHELL (d);
-
-    XtSetArg (al [0], XtNmappedWhenManaged, False);
-    XtSetArg (al [1], XtNwidth,  1);
-    XtSetArg (al [2], XtNheight, 1);
-    XtSetValues (shell, al, 3);
-    XtRealizeWidget (shell);
-
-    {
-      int new_argc;
-      char **new_argv;
-      make_argc_argv (Vcommand_line_args, &new_argc, &new_argv);
-      XSetCommand (XtDisplay (shell), XtWindow (shell), new_argv, new_argc);
-      free_argc_argv (new_argv);
-    }
-
+    int new_argc;
+    char **new_argv;
+    make_argc_argv (Vcommand_line_args, &new_argc, &new_argv);
+    XSetCommand (XtDisplay (app_shell), XtWindow (app_shell), new_argv, new_argc);
+    free_argc_argv (new_argv);
   }
 #endif /* HAVE_SESSION */
 
+
 #ifdef HAVE_OFFIX_DND
-  DndInitialize ( DEVICE_XT_APP_SHELL (d) );
+  DndInitialize ( app_shell );
 #endif
 
   Vx_initial_argv_list = make_arg_list (argc, argv);
@@ -340,8 +576,7 @@ x_init_device (struct device *d, Lisp_Object props)
   init_baud_rate (d);
   init_one_device (d);
 
-  DEVICE_X_GC_CACHE (d) =
-    make_gc_cache (dpy, RootWindow (dpy, DefaultScreen (dpy)));
+  DEVICE_X_GC_CACHE (d) = make_gc_cache (dpy, XtWindow(app_shell));
   DEVICE_X_GRAY_PIXMAP (d) = None;
   Xatoms_of_device_x (d);
   Xatoms_of_xselect (d);
@@ -437,51 +672,51 @@ x_delete_device (struct device *d)
 /*				handle X errors				*/
 /************************************************************************/
 
-static CONST char *events[] =
-{
-   "0: ERROR!",
-   "1: REPLY",
-   "KeyPress",
-   "KeyRelease",
-   "ButtonPress",
-   "ButtonRelease",
-   "MotionNotify",
-   "EnterNotify",
-   "LeaveNotify",
-   "FocusIn",
-   "FocusOut",
-   "KeymapNotify",
-   "Expose",
-   "GraphicsExpose",
-   "NoExpose",
-   "VisibilityNotify",
-   "CreateNotify",
-   "DestroyNotify",
-   "UnmapNotify",
-   "MapNotify",
-   "MapRequest",
-   "ReparentNotify",
-   "ConfigureNotify",
-   "ConfigureRequest",
-   "GravityNotify",
-   "ResizeRequest",
-   "CirculateNotify",
-   "CirculateRequest",
-   "PropertyNotify",
-   "SelectionClear",
-   "SelectionRequest",
-   "SelectionNotify",
-   "ColormapNotify",
-   "ClientMessage",
-   "MappingNotify",
-   "LASTEvent"
-};
-
 CONST char *
 x_event_name (int event_type)
 {
-  if (event_type < 0) return 0;
-  if (event_type >= countof (events)) return 0;
+  static CONST char *events[] =
+  {
+    "0: ERROR!",
+    "1: REPLY",
+    "KeyPress",
+    "KeyRelease",
+    "ButtonPress",
+    "ButtonRelease",
+    "MotionNotify",
+    "EnterNotify",
+    "LeaveNotify",
+    "FocusIn",
+    "FocusOut",
+    "KeymapNotify",
+    "Expose",
+    "GraphicsExpose",
+    "NoExpose",
+    "VisibilityNotify",
+    "CreateNotify",
+    "DestroyNotify",
+    "UnmapNotify",
+    "MapNotify",
+    "MapRequest",
+    "ReparentNotify",
+    "ConfigureNotify",
+    "ConfigureRequest",
+    "GravityNotify",
+    "ResizeRequest",
+    "CirculateNotify",
+    "CirculateRequest",
+    "PropertyNotify",
+    "SelectionClear",
+    "SelectionRequest",
+    "SelectionNotify",
+    "ColormapNotify",
+    "ClientMessage",
+    "MappingNotify",
+    "LASTEvent"
+  };
+
+  if (event_type < 0 || event_type >= countof (events))
+    return NULL;
   return events [event_type];
 }
 
@@ -792,21 +1027,8 @@ construct_name_list (Display *display, Widget widget, char *fake_name,
 
 #endif /* 0 */
 
-/* Only the characters [-_A-Za-z0-9] are allowed in the individual
-   sections of a resource.  Convert invalid characters to -. */
-
-static void
-validify_resource_string (char *str)
-{
-  while (*str)
-    {
-      if (!strchr ("ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-		   "abcdefghijklmnopqrstuvwxyz"
-		   "0123456789-_", *str))
-	*str = '-';
-      str++;
-    }
-}
+static char_dynarr *name_char_dynarr;
+static char_dynarr *class_char_dynarr;
 
 /* Given a locale and device specification from x-get-resource or
 x-get-resource-prefix, return the resource prefix and display to
@@ -814,11 +1036,9 @@ fetch the resource on. */
 
 static void
 x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
-		       Display **display_out, char *name_out,
-		       char *class_out)
+		       Display **display_out, char_dynarr *name,
+		       char_dynarr *class)
 {
-  char *appname, *appclass;
-
   if (NILP (locale))
     locale = Qglobal;
   if (NILP (Fvalid_specifier_locale_p (locale)))
@@ -850,35 +1070,41 @@ x_get_resource_prefix (Lisp_Object locale, Lisp_Object device,
 
   *display_out = DEVICE_X_DISPLAY (XDEVICE (device));
 
-  XtGetApplicationNameAndClass (*display_out, &appname, &appclass);
-  strcpy (name_out, appname);
-  strcpy (class_out, appclass);
-  validify_resource_string (name_out);
-  validify_resource_string (class_out);
+  {
+    char *appname, *appclass;
+    int name_len, class_len;
+    XtGetApplicationNameAndClass (*display_out, &appname, &appclass);
+    name_len  = strlen (appname);
+    class_len = strlen (appclass);
+    Dynarr_add_many (name , appname,  name_len);
+    Dynarr_add_many (class, appclass, class_len);
+    validify_resource_component (Dynarr_atp (name,  0), name_len);
+    validify_resource_component (Dynarr_atp (class, 0), class_len);
+  }
 
   if (EQ (locale, Qglobal))
     return;
   if (BUFFERP (locale))
     {
-      strcat (name_out, ".buffer.");
+      Dynarr_add_literal_string (name, ".buffer.");
       /* we know buffer is live; otherwise we got an error above. */
-      strcat (name_out, (CONST char *) XSTRING_DATA (Fbuffer_name (locale)));
-      strcat (class_out, ".EmacsLocaleType.EmacsBuffer");
+      Dynarr_add_validified_lisp_string (name, Fbuffer_name (locale));
+      Dynarr_add_literal_string (class, ".EmacsLocaleType.EmacsBuffer");
     }
   else if (FRAMEP (locale))
     {
-      strcat (name_out, ".frame.");
+      Dynarr_add_literal_string (name, ".frame.");
       /* we know frame is live; otherwise we got an error above. */
-      strcat (name_out, (CONST char *) XSTRING_DATA (Fframe_name (locale)));
-      strcat (class_out, ".EmacsLocaleType.EmacsFrame");
+      Dynarr_add_validified_lisp_string (name, Fframe_name (locale));
+      Dynarr_add_literal_string (class, ".EmacsLocaleType.EmacsFrame");
     }
   else
     {
       assert (DEVICEP (locale));
-      strcat (name_out, ".device.");
+      Dynarr_add_literal_string (name, ".device.");
       /* we know device is live; otherwise we got an error above. */
-      strcat (name_out, (CONST char *) XSTRING_DATA (Fdevice_name (locale)));
-      strcat (class_out, ".EmacsLocaleType.EmacsDevice");
+      Dynarr_add_validified_lisp_string (name, Fdevice_name (locale));
+      Dynarr_add_literal_string (class, ".EmacsLocaleType.EmacsDevice");
     }
   return;
 }
@@ -887,11 +1113,11 @@ DEFUN ("x-get-resource", Fx_get_resource, 3, 6, 0, /*
 Retrieve an X resource from the resource manager.
 
 The first arg is the name of the resource to retrieve, such as "font".
-The second arg is the class of the resource to retrieve, like "Font".
-The third arg should be one of the symbols 'string, 'integer, 'natnum, or
+The second arg is the class of the resource to retrieve, such as "Font".
+The third arg must be one of the symbols 'string, 'integer, 'natnum, or
   'boolean, specifying the type of object that the database is searched for.
 The fourth arg is the locale to search for the resources on, and can
-  currently be a a buffer, a frame, a device, or 'global.  If omitted, it
+  currently be a buffer, a frame, a device, or 'global.  If omitted, it
   defaults to 'global.
 The fifth arg is the device to search for the resources on. (The resource
   database for a particular device is constructed by combining non-device-
@@ -967,8 +1193,7 @@ mean ``unspecified.''
 */
        (name, class, type, locale, device, no_error))
 {
-  /* #### fixed limit, could be overflowed */
-  char name_string[2048], class_string[2048];
+  char* name_string, *class_string;
   char *raw_result;
   XrmDatabase db;
   Display *display;
@@ -978,28 +1203,25 @@ mean ``unspecified.''
   CHECK_STRING (class);
   CHECK_SYMBOL (type);
 
-  if (!EQ (type, Qstring)  &&
-      !EQ (type, Qboolean) &&
-      !EQ (type, Qinteger) &&
-      !EQ (type, Qnatnum))
-    return maybe_signal_continuable_error
-      (Qwrong_type_argument,
-       list2 (build_translated_string
-	      ("should be string, integer, natnum or boolean"),
-	      type),
-       Qresource, errb);
+  Dynarr_reset (name_char_dynarr);
+  Dynarr_reset (class_char_dynarr);
 
-  x_get_resource_prefix (locale, device, &display, name_string,
-			 class_string);
+  x_get_resource_prefix (locale, device, &display,
+			 name_char_dynarr, class_char_dynarr);
   if (!display)
     return Qnil;
 
   db = XtDatabase (display);
 
-  strcat (name_string, ".");
-  strcat (name_string, (CONST char *) XSTRING_DATA (name));
-  strcat (class_string, ".");
-  strcat (class_string, (CONST char *) XSTRING_DATA (class));
+  Dynarr_add (name_char_dynarr, '.');
+  Dynarr_add_lisp_string (name_char_dynarr, name);
+  Dynarr_add (class_char_dynarr, '.');
+  Dynarr_add_lisp_string (class_char_dynarr, class);
+  Dynarr_add (name_char_dynarr,  '\0');
+  Dynarr_add (class_char_dynarr, '\0');
+
+  name_string  = Dynarr_atp (name_char_dynarr,  0);
+  class_string = Dynarr_atp (class_char_dynarr, 0);
 
   {
     XrmValue xrm_value;
@@ -1035,14 +1257,13 @@ mean ``unspecified.''
 	  !strcasecmp (raw_result, "false") ||
 	  !strcasecmp (raw_result, "no"))
 	return Fcons (Qnil, Qnil);
-      else if (!strcasecmp (raw_result, "on")   ||
-	       !strcasecmp (raw_result, "true") ||
-	       !strcasecmp (raw_result, "yes"))
+      if (!strcasecmp (raw_result, "on")   ||
+	  !strcasecmp (raw_result, "true") ||
+	  !strcasecmp (raw_result, "yes"))
 	return Fcons (Qt, Qnil);
-      else
-	return maybe_continuable_error (Qresource, errb,
-					"can't convert %s: %s to a Boolean",
-					name_string, raw_result);
+      return maybe_continuable_error
+	(Qresource, errb,
+	 "can't convert %s: %s to a Boolean", name_string, raw_result);
     }
   else if (EQ (type, Qinteger) || EQ (type, Qnatnum))
     {
@@ -1051,21 +1272,23 @@ mean ``unspecified.''
       if (1 != sscanf (raw_result, "%d%c", &i, &c))
 	return maybe_continuable_error
 	  (Qresource, errb,
-	   "can't convert %s: %s to an integer",
-	   name_string, raw_result);
+	   "can't convert %s: %s to an integer", name_string, raw_result);
       else if (EQ (type, Qnatnum) && i < 0)
 	return maybe_continuable_error
 	  (Qresource, errb,
-	   "invalid numerical value %d for resource %s",
-	   i, name_string);
+	   "invalid numerical value %d for resource %s", i, name_string);
       else
 	return make_int (i);
     }
   else
-    abort ();
-
-  /* Can't get here. */
-  return Qnil;	/* shut up compiler */
+    {
+      return maybe_signal_continuable_error
+	(Qwrong_type_argument,
+	 list2 (build_translated_string
+		("should be string, integer, natnum or boolean"),
+		type),
+	 Qresource, errb);
+    }
 }
 
 DEFUN ("x-get-resource-prefix", Fx_get_resource_prefix, 1, 2, 0, /*
@@ -1080,14 +1303,20 @@ returns nil. (In such a case, `x-get-resource' would always return nil.)
 */
        (locale, device))
 {
-  /* #### fixed limit, could be overflowed */
-  char name[1024], class[1024];
   Display *display;
 
-  x_get_resource_prefix (locale, device, &display, name, class);
+  Dynarr_reset (name_char_dynarr );
+  Dynarr_reset (class_char_dynarr);
+
+  x_get_resource_prefix (locale, device, &display,
+			 name_char_dynarr, class_char_dynarr);
   if (!display)
     return Qnil;
-  return Fcons (build_string (name), build_string (class));
+
+  return Fcons (make_string ((Bufbyte *) Dynarr_atp (name_char_dynarr, 0),
+			     Dynarr_length (name_char_dynarr)),
+		make_string ((Bufbyte *) Dynarr_atp (class_char_dynarr, 0),
+			     Dynarr_length (class_char_dynarr)));
 }
 
 DEFUN ("x-put-resource", Fx_put_resource, 1, 2, 0, /*
@@ -1108,7 +1337,8 @@ standard resource specification.
   if (strspn (str,
 	      /* Only the following chars are allowed before the colon */
 	      " \t.*?abcdefghijklmnopqrstuvwxyz"
-	      "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-") != colon_pos - str)
+	      "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-")
+      != (size_t) (colon_pos - str))
     goto invalid;
 
   if (DEVICE_X_P (d))
@@ -1135,14 +1365,15 @@ This is the first-created X device that still exists.
 }
 
 DEFUN ("x-display-visual-class", Fx_display_visual_class, 0, 1, 0, /*
-Return the visual class of the X display `device' is on.
+Return the visual class of the X display DEVICE is using.
+This can be altered from the default at startup using the XResource "EmacsVisual".
 The returned value will be one of the symbols `static-gray', `gray-scale',
 `static-color', `pseudo-color', `true-color', or `direct-color'.
 */
        (device))
 {
-  switch (DefaultVisualOfScreen
-	  (DefaultScreenOfDisplay (get_x_display (device)))->class)
+  Visual *vis = DEVICE_X_VISUAL (decode_x_device (device));
+  switch (vis->class)
     {
     case StaticGray:  return intern ("static-gray");
     case GrayScale:   return intern ("gray-scale");
@@ -1152,61 +1383,44 @@ The returned value will be one of the symbols `static-gray', `gray-scale',
     case DirectColor: return intern ("direct-color");
     default:
       error ("display has an unknown visual class");
+      return Qnil;	/* suppress compiler warning */
     }
-
-  return Qnil;	/* suppress compiler warning */
 }
 
-static int
-x_device_pixel_width (struct device *d)
+DEFUN ("x-display-visual-depth", Fx_display_visual_depth, 0, 1, 0, /*
+Return the bitplane depth of the visual the X display DEVICE is using.
+*/
+       (device))
+{
+   return make_int (DEVICE_X_DEPTH (decode_x_device (device)));
+}
+
+static Lisp_Object
+x_device_system_metrics (struct device *d,
+			 enum device_metrics m)
 {
   Display *dpy = DEVICE_X_DISPLAY (d);
 
-  return DisplayWidth (dpy, DefaultScreen (dpy));
-}
-
-static int
-x_device_pixel_height (struct device *d)
-{
-  Display *dpy = DEVICE_X_DISPLAY (d);
-
-  return DisplayHeight (dpy, DefaultScreen (dpy));
-}
-
-static int
-x_device_mm_width (struct device *d)
-{
-  Display *dpy = DEVICE_X_DISPLAY (d);
-
-  return DisplayWidthMM (dpy, DefaultScreen (dpy));
-}
-
-static int
-x_device_mm_height (struct device *d)
-{
-  Display *dpy = DEVICE_X_DISPLAY (d);
-
-  return DisplayHeightMM (dpy, DefaultScreen (dpy));
-}
-
-static int
-x_device_bitplanes (struct device *d)
-{
-  Display *dpy = DEVICE_X_DISPLAY (d);
-
-  return DisplayPlanes (dpy, DefaultScreen (dpy));
-}
-
-static int
-x_device_color_cells (struct device *d)
-{
-  Display *dpy = DEVICE_X_DISPLAY (d);
-
-  return DisplayCells (dpy, DefaultScreen (dpy));
+  switch (m)
+    {
+    case DM_size_device:
+      return Fcons (make_int (DisplayWidth (dpy, DefaultScreen (dpy))),
+		    make_int (DisplayHeight (dpy, DefaultScreen (dpy))));
+    case DM_size_device_mm:
+      return Fcons (make_int (DisplayWidthMM (dpy, DefaultScreen (dpy))),
+		    make_int (DisplayHeightMM (dpy, DefaultScreen (dpy))));
+    case DM_num_bit_planes:
+      return make_int (DisplayPlanes (dpy, DefaultScreen (dpy)));
+    case DM_num_color_cells:
+      return make_int (DisplayCells (dpy, DefaultScreen (dpy)));
+    default: /* No such device metric property for X devices  */
+      return Qunbound;
+    }
 }
 
 DEFUN ("x-server-vendor", Fx_server_vendor, 0, 1, 0, /*
-Return the vendor ID string of the X server `device' on.
+Return the vendor ID string of the X server DEVICE is on.
+Return the empty string if the vendor ID string cannot be determined.
 */
        (device))
 {
@@ -1217,7 +1431,7 @@ Return the vendor ID string of the X server `device' on.
 }
 
 DEFUN ("x-server-version", Fx_server_version, 0, 1, 0, /*
-Return the version numbers of the X server `device' is on.
+Return the version numbers of the X server DEVICE is on.
 The returned value is a list of three integers: the major and minor
 version numbers of the X Protocol in use, and the vendor-specific release
 number.  See also `x-server-vendor'.
@@ -1259,8 +1473,8 @@ name keys on the keyboard.  See `x-keysym-on-keyboard-p'.
   return DEVICE_X_DATA (d)->x_keysym_map_hashtable;
 }
 
-DEFUN ("x-keysym-on-keyboard-sans-modifiers-p",
-       Fx_keysym_on_keyboard_sans_modifiers_p, 1, 2, 0, /*
+DEFUN ("x-keysym-on-keyboard-sans-modifiers-p", Fx_keysym_on_keyboard_sans_modifiers_p,
+       1, 2, 0, /*
 Return true if KEYSYM names a key on the keyboard of DEVICE.
 More precisely, return true if pressing a physical key
 on the keyboard of DEVICE without any modifier keys generates KEYSYM.
@@ -1448,6 +1662,7 @@ syms_of_device_x (void)
 
   DEFSUBR (Fdefault_x_device);
   DEFSUBR (Fx_display_visual_class);
+  DEFSUBR (Fx_display_visual_depth);
   DEFSUBR (Fx_server_vendor);
   DEFSUBR (Fx_server_version);
   DEFSUBR (Fx_valid_keysym_name_p);
@@ -1472,12 +1687,18 @@ console_type_create_device_x (void)
   CONSOLE_HAS_METHOD (x, finish_init_device);
   CONSOLE_HAS_METHOD (x, mark_device);
   CONSOLE_HAS_METHOD (x, delete_device);
-  CONSOLE_HAS_METHOD (x, device_pixel_width);
-  CONSOLE_HAS_METHOD (x, device_pixel_height);
-  CONSOLE_HAS_METHOD (x, device_mm_width);
-  CONSOLE_HAS_METHOD (x, device_mm_height);
-  CONSOLE_HAS_METHOD (x, device_bitplanes);
-  CONSOLE_HAS_METHOD (x, device_color_cells);
+  CONSOLE_HAS_METHOD (x, device_system_metrics);
+
+  {
+    /* Initialize variables to speed up X resource interactions */
+    CONST char *valid_resource_chars =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
+    while (*valid_resource_chars)
+      valid_resource_char_p[(unsigned int) (*valid_resource_chars++)] = 1;
+
+    name_char_dynarr  = Dynarr_new (char);
+    class_char_dynarr = Dynarr_new (char);
+  }
 }
 
 void
@@ -1490,8 +1711,17 @@ that XEmacs will use.  For changes to this variable to take effect, they
 must be made before the connection to the X server is initialized, that is,
 this variable may only be changed before emacs is dumped, or by setting it
 in the file lisp/term/x-win.el.
+
+If this variable is nil before the connection to the X server is first
+initialized (which it is by default), the X resource database will be
+consulted and the value will be set according to whether any resources
+are found for the application class `XEmacs'.  If the user has set any
+resources for the XEmacs application class, the XEmacs process will use
+the application class `XEmacs'.  Otherwise, the XEmacs process will use
+the application class `Emacs' which is backwards compatible to previous
+XEmacs versions but may conflict with resources intended for GNU Emacs.
 */ );
-  Vx_emacs_application_class = Fpurecopy (build_string ("Emacs"));
+  Vx_emacs_application_class = Qnil;
 
   DEFVAR_LISP ("x-initial-argv-list", &Vx_initial_argv_list /*
 You don't want to know.
@@ -1505,13 +1735,21 @@ just reside in C.
   Vx_initial_argv_list = Qnil;
 
   DEFVAR_BOOL ("x-seppuku-on-epipe", &x_seppuku_on_epipe /*
-When non-nil terminate XEmacs immediately on SIGPIPE from the X server.
+When non-nil, terminate XEmacs immediately on SIGPIPE from the X server.
 XEmacs doesn't terminate properly on some systems.
 When this variable is non-nil, XEmacs will commit immediate suicide
 when it gets a sigpipe from the X Server.
 */ );
   x_seppuku_on_epipe = 0;
 
+#if defined(MULE) && (defined(LWLIB_MENUBARS_MOTIF) || defined(HAVE_XIM) || defined (USE_XFONTSET))
+  DEFVAR_LISP ("x-app-defaults-directory", &Vx_app_defaults_directory /*
+Used by the Lisp code to communicate to the low level X initialization
+where the localized init files are.
+*/ );
+  Vx_app_defaults_directory = Qnil;
+#endif
+
   Fprovide (Qx);
 
   staticpro (&Vdefault_x_device);
@@ -1521,5 +1759,4 @@ when it gets a sigpipe from the X Server.
   error_occurred = 0;
 
   in_resource_setting = 0;
-  in_specifier_change_function = 0;
 }
