@@ -114,6 +114,10 @@ Boston, MA 02111-1307, USA.  */
  unselect_device_cb	(those that use select() and file descriptors and
 			have a separate input fd per device).
 
+ create_stream_pair_cb  These callbacks are called by process code to
+ delete_stream_pair_cb  create and delete a pait of input and output lstreams
+			which are used for subprocess I/O.
+
  quitp_cb		A handler function called from the `QUIT' macro which
 			should check whether the quit character has been
 			typed.  On systems with SIGIO, this will not be called
@@ -156,10 +160,7 @@ Boston, MA 02111-1307, USA.  */
 			   command builder) will get affected.
 			-- for timer, process, magic-eval, and eval events,
 			   channel will be nil.
-			-- for scrollbar misc-user events, channel
-			   will be a window.
-			-- for menubar misc-user events, channel
-			   will be a frame.
+			-- for misc-user events, channel will be a frame.
 			-- for magic events, channel will be a frame
 			   (usually) or a device.
 
@@ -223,13 +224,19 @@ Boston, MA 02111-1307, USA.  */
     function		An elisp function to call with this event object.
     internal_function	Ignored.
     object		Anything.
+    button		What button went down or up.
+    modifiers		Bucky-bits on that button: shift, control, meta, etc.
+    x, y		Where it was at the button-state-change (in pixels).
 			This is similar to an eval_event, except that it is
 			generated by user actions: selections in the
-			menubar or scrollbar actions.  It is a "command"
-			event, like key and mouse presses (and unlike mouse
-			motion, process output, and enter and leave window
-			hooks).  In many ways, eval_events are not the same
-			as keypresses or misc_user_events.
+			menubar, scrollbar actions, or drag and drop actions.
+			It is a "command" event, like key and mouse presses
+			(and unlike mouse motion, process output, and enter
+			and leave window hooks).  In many ways, eval_events
+			are not the same as keypresses or misc_user_events.
+			The button, modifiers, x, and y parts are only used
+			by the XEmacs Drag'n'Drop system. Don't depend on their
+			values for other types of misc_user_events.
 
  magic_event
 			No user-serviceable parts within.  This is for things
@@ -255,18 +262,58 @@ Boston, MA 02111-1307, USA.  */
 			a magic_event; the Lisp programmer need not know
 			anything more.
 
- #ifdef HAVE_OFFIX_DND
- dnd_drop_event
- dnd_drag_event (* to be implemented *)
-    button		What button went down or up.
-    modifiers		Bucky-bits on that button: shift, control, meta, etc.
-    x, y		Where it was at the button-state-change (in pixels).
-    data                A list of type and data.
-                        This events are created by Drag'n'Drop actions.
-			Currently only the OffiX Dnd Protocol is supported.
- #endif
+*/
 
- */
+/*
+  Stream pairs description
+  ------------------------
+
+  Since there are many possible processes/event loop combinations, the event code
+  is responsible for creating an appropriare lstream type. The process
+  implementation does not care about that implementation.
+
+  The Create stream pair function is passed two void* values, which identify
+  process-dependant 'handles'. The process implementation uses these handles
+  to communicate with child processes. The function must be prepared to receive
+  handle types of any process implementation. Since there only one process
+  implementation exists in a particular XEmacs configuration, preprocessing
+  is a mean of compiling in the support for the code which deals with particular
+  handle types.
+
+  For example, a unixoid type loop, which relies on file descriptors, may be
+  asked to create a pair of streams by a unix-style process implementation.
+  In this case, the handles passed are unix file descriptors, and the code
+  may deal with these directly. Although, the same code may be used on Win32
+  system with X-Windows. In this case, Win32 process implementation passes
+  handles of type HANDLE, and the create_stream_pair function must call
+  appropriate function to get file descriptors given HANDLEs, so that these
+  descriptors may be passed to XtAddInput.
+
+  The handle given may have special denying value, in which case the
+  corresponding lstream should not be created.
+
+  The return value of the function is a unique stream identifier. It is used
+  by processes implementation, in its  platform-independant part. There is
+  the get_process_from_usid function, which returns process object given its
+  USID. The event stream is responsible for converting its internal handle
+  type into USID.
+
+  Example is the TTY event stream. When a file descriptor signals input, the
+  event loop must determine process to which the input is destined. Thus,
+  the imlementation uses process input stream file descriptor as USID, by
+  simply casting the fd value to USID type.
+
+  There are two special USID values. One, USID_ERROR, indicates that the stream
+  pair cannot be created. The second, USID_DONTHASH, indicates that streams are
+  created, but the event stream does not wish to be able to find the process
+  by its USID. Specifically, if an event stream implementation never calss
+  get_process_from_usid, this value should always be returned, to prevent
+  accumulating useless information on USID to process relationship.
+*/
+
+/* typedef unsigned int USID; in lisp.h */
+#define USID_ERROR ((USID)-1)
+#define USID_DONTHASH ((USID)0)
 
 
 struct Lisp_Event;
@@ -284,8 +331,17 @@ struct event_stream
   void (*select_process_cb)	(struct Lisp_Process *);
   void (*unselect_process_cb)	(struct Lisp_Process *);
   void (*quit_p_cb)		(void);
+  USID (*create_stream_pair_cb) (void* /* inhandle*/, void* /*outhandle*/ ,
+				 Lisp_Object* /* instream */,
+				 Lisp_Object* /* outstream */,
+				 int /* flags */);
+  USID (*delete_stream_pair_cb) (Lisp_Object /* instream */,
+				 Lisp_Object /* outstream */);
 };
 
+/* Flags for create_stream_pair_cb() FLAGS parameter */
+#define STREAM_PTY_FLUSHING		0x0001
+#define STREAM_NETWORK_CONNECTION	0x0002
 
 extern struct event_stream *event_stream;
 
@@ -302,9 +358,6 @@ typedef enum emacs_event_type
   magic_eval_event,
   eval_event,
   misc_user_event,
-#ifdef HAVE_OFFIX_DND
-  dnd_drop_event,
-#endif
   dead_event
 } emacs_event_type;
 
@@ -349,31 +402,35 @@ struct eval_data
   Lisp_Object	    object;
 };
 
+struct misc_user_data
+{
+  Lisp_Object       function;
+  Lisp_Object	    object;
+  int               button;
+  unsigned char     modifiers;
+  int               x, y;
+};
+
 struct magic_eval_data
 {
   void		    (*internal_function) (Lisp_Object);
   Lisp_Object	    object;
 };
 
-#ifdef HAVE_OFFIX_DND
-struct dnd_drop_data
-{
-  int               button;
-  unsigned char     modifiers;
-  int               x, y;
-  Lisp_Object       data;
-};
-#endif
-
 #if defined (HAVE_X_WINDOWS) && defined(emacs)
 # include <X11/Xlib.h>
 #endif
 
 union magic_data
 {
-  char             underlying_tty_event;
+#ifdef HAVE_TTY
+  char		    underlying_tty_event;
+#endif
 #ifdef HAVE_X_WINDOWS
-  XEvent           underlying_x_event;
+  XEvent	    underlying_x_event;
+#endif
+#ifdef HAVE_MS_WINDOWS
+  int		    underlying_mswindows_event;
 #endif
 };
 
@@ -397,12 +454,10 @@ struct Lisp_Event
       struct motion_data	motion;
       struct process_data	process;
       struct timeout_data	timeout;
-      struct eval_data		eval;	/* misc_user_event uses this too */
+      struct eval_data		eval;	/* misc_user_event no loger uses this */
+      struct misc_user_data	misc;	/* because it needs position information */
       union magic_data		magic;
       struct magic_eval_data	magic_eval;
-#ifdef HAVE_OFFIX_DND
-      struct dnd_drop_data      dnd_drop;
-#endif
     } event;
 };
 
@@ -428,19 +483,32 @@ DECLARE_LRECORD (command_builder, struct command_builder);
 
 #define EVENT_LIVE_P(a) (EVENT_TYPE (a) != dead_event)
 
-#define CHECK_LIVE_EVENT(x)						\
-  do { CHECK_EVENT (x);							\
-       if (! EVENTP (x)							\
-	   || ! EVENT_LIVE_P (XEVENT (x)))				\
-         dead_wrong_type_argument (Qevent_live_p, (x)); } while (0)
-#define CONCHECK_LIVE_EVENT(x)						\
-  do { CONCHECK_EVENT (x);						\
-       if (! EVENTP (x)							\
-	   || ! EVENT_LIVE_P (XEVENT (x)))				\
-         x = wrong_type_argument (Qevent_live_p, (x)); } while (0)
+#define CHECK_LIVE_EVENT(x) do {			\
+  CHECK_EVENT (x);					\
+  if (! EVENT_LIVE_P (XEVENT (x)))			\
+    dead_wrong_type_argument (Qevent_live_p, (x));	\
+} while (0)
+#define CONCHECK_LIVE_EVENT(x) do {			\
+  CONCHECK_EVENT (x);					\
+  if (! EVENT_LIVE_P (XEVENT (x)))			\
+    x = wrong_type_argument (Qevent_live_p, (x));	\
+} while (0)
 
 
-extern Lisp_Object Qevent_live_p;
+EXFUN (Fcharacter_to_event, 4);
+EXFUN (Fdeallocate_event, 1);
+EXFUN (Fevent_glyph_extent, 1);
+EXFUN (Fevent_modeline_position, 1);
+EXFUN (Fevent_over_modeline_p, 1);
+EXFUN (Fevent_over_toolbar_p, 1);
+EXFUN (Fevent_over_vertical_divider_p, 1);
+EXFUN (Fevent_point, 1);
+EXFUN (Fevent_window, 1);
+EXFUN (Fmake_event, 2);
+
+extern Lisp_Object QKbackspace, QKdelete, QKescape, QKlinefeed, QKreturn;
+extern Lisp_Object QKspace, QKtab, Qmouse_event_p, Vcharacter_set_property;
+extern Lisp_Object Qcancel_mode_internal;
 
 /* Note: under X Windows, MOD_ALT is generated by the Alt key if there are
    both Alt and Meta keys.  If there are no Meta keys, then Alt generates
@@ -456,7 +524,8 @@ Lisp_Object allocate_command_builder (Lisp_Object console);
 void format_event_object (char *buf, struct Lisp_Event *e, int brief);
 void character_to_event (Emchar c, struct Lisp_Event *event,
 			 struct console *con,
-			 int use_console_meta_flag);
+			 int use_console_meta_flag,
+			 int do_backspace_mapping);
 void enqueue_magic_eval_event (void (*fun) (Lisp_Object), Lisp_Object object);
 void zero_event (struct Lisp_Event *e);
 
@@ -480,13 +549,15 @@ int command_event_p (Lisp_Object event);
 
 struct console *event_console_or_selected (Lisp_Object event);
 
-int event_stream_event_pending_p (int user);
 void event_stream_next_event (struct Lisp_Event *event);
 void event_stream_handle_magic_event (struct Lisp_Event *event);
 void event_stream_select_console   (struct console *c);
 void event_stream_unselect_console (struct console *c);
 void event_stream_select_process   (struct Lisp_Process *proc);
 void event_stream_unselect_process (struct Lisp_Process *proc);
+USID event_stream_create_stream_pair (void* inhandle, void* outhandle,
+		Lisp_Object* instream, Lisp_Object* outstream, int flags);
+USID event_stream_delete_stream_pair (Lisp_Object instream, Lisp_Object outstream);
 void event_stream_quit_p (void);
 
 struct low_level_timeout
@@ -516,6 +587,10 @@ void event_stream_deal_with_async_timeout (int interval_id);
 int event_stream_add_async_timeout (EMACS_TIME thyme);
 void event_stream_remove_async_timeout (int id);
 
+/* from event-stream.c -- focus sanity */
+extern int focus_follows_mouse;
+void investigate_frame_change (void);
+
 void emacs_handle_focus_change_preliminary (Lisp_Object frame_inp_and_dev);
 void emacs_handle_focus_change_final (Lisp_Object frame_inp_and_dev);
 
@@ -541,12 +616,20 @@ int event_stream_unixoid_select_process   (struct Lisp_Process *proc);
 int event_stream_unixoid_unselect_process (struct Lisp_Process *proc);
 int read_event_from_tty_or_stream_desc (struct Lisp_Event *event,
 					struct console *c, int fd);
+USID event_stream_unixoid_create_stream_pair (void* inhandle, void* outhandle,
+					     Lisp_Object* instream,
+					     Lisp_Object* outstream,
+					     int flags);
+USID event_stream_unixoid_delete_stream_pair (Lisp_Object instream,
+					      Lisp_Object outstream);
+
+/* Beware: this evil macro evaluates its arg many times */
+#define FD_TO_USID(fd) ((fd)==0 ? (USID)999999 : ((fd)<0 ? USID_DONTHASH : (USID)(fd)))
+
 #endif /* HAVE_UNIXOID_EVENT_LOOP */
 
 extern int emacs_is_blocking;
 
-extern Lisp_Object Vcontrolling_terminal;
-
 extern volatile int sigint_happened;
 
 /* Define this if you want the tty event stream to be used when the
