@@ -34,6 +34,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "buffer.h"
 #include "commands.h"           /* zmacs_regions */
+#include "gui.h"
 #include "events.h"
 #include "frame.h"
 #include "opaque.h"
@@ -123,6 +124,11 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 	{
 	  wv->name = string_chars;
 	  wv->enabled = 1;
+	  /* dverna Dec. 98: command_builder_operate_menu_accelerator will
+	     manipulate the accel as a Lisp_Object if the widget has a name.
+	     Since simple labels have a name, but no accel, we *must* set it
+	     to nil */
+	  wv->accel = LISP_TO_VOID (Qnil);
 	}
     }
   else if (VECTORP (desc))
@@ -145,8 +151,10 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 	{
 	  Lisp_Object key, val;
 	  Lisp_Object include_p = Qnil, hook_fn = Qnil, config_tag = Qnil;
+	  Lisp_Object active_p = Qt;
 	  Lisp_Object accel;
 	  int included_spec = 0;
+	  int active_spec = 0;
 	  wv->type = CASCADE_TYPE;
 	  wv->enabled = 1;
 	  wv->name = (char *) XSTRING_DATA (LISP_GETTEXT (XCAR (desc)));
@@ -171,6 +179,8 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 		config_tag = val;
 	      else if (EQ (key, Q_filter))
 		hook_fn = val;
+	      else if (EQ (key, Q_active))
+		active_p = val, active_spec = 1;
 	      else if (EQ (key, Q_accelerator))
 		{
 		  if ( SYMBOLP (val)
@@ -179,6 +189,10 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 		  else
 		    signal_simple_error ("bad keyboard accelerator", val);
 		}
+	      else if (EQ (key, Q_label))
+		{
+		  /* implement in 21.2 */
+		}
 	      else
 		signal_simple_error ("unknown menu cascade keyword", cascade);
 	    }
@@ -190,9 +204,13 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 	      wv = NULL;
 	      goto menu_item_done;
 	    }
-	  if (!NILP (hook_fn))
+
+	  if (active_spec)
+	    active_p = Feval (active_p);
+	  
+	  if (!NILP (hook_fn) && !NILP (active_p))
 	    {
-#ifdef LWLIB_MENUBARS_LUCID
+#if defined LWLIB_MENUBARS_LUCID || defined LWLIB_MENUBARS_MOTIF
 	      if (filter_p || depth == 0)
 		{
 #endif
@@ -200,7 +218,7 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 						hook_fn, desc);
 		  if (UNBOUNDP (desc))
 		    desc = Qnil;
-#ifdef LWLIB_MENUBARS_LUCID
+#if defined LWLIB_MENUBARS_LUCID || defined LWLIB_MENUBARS_MOTIF
 		}
 	      else
 		{
@@ -215,7 +233,7 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 		  incr_wv->call_data = LISP_TO_VOID (incremental_data);
 		  goto menu_item_done;
 		}
-#endif /* LWLIB_MENUBARS_LUCID */
+#endif /* LWLIB_MENUBARS_LUCID || LWLIB_MENUBARS_MOTIF */
 	    }
 	  if (menu_type == POPUP_TYPE && popup_menu_titles && depth == 0)
 	    {
@@ -235,6 +253,24 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 	      wv->contents = title_wv;
 	      prev = sep_wv;
 	    }
+	  wv->enabled = ! NILP (active_p);
+	  if (deep_p && !wv->enabled  && !NILP (desc))
+	    {
+	      widget_value *dummy;
+	      /* Add a fake entry so the menus show up */
+	      wv->contents = dummy = xmalloc_widget_value ();
+	      dummy->name = "(inactive)";
+	      dummy->accel = LISP_TO_VOID (Qnil);
+	      dummy->enabled = 0;
+	      dummy->selected = 0;
+	      dummy->value = NULL;
+	      dummy->type = BUTTON_TYPE;
+	      dummy->call_data = NULL;
+	      dummy->next = NULL;
+	      
+	      goto menu_item_done;
+	}
+
 	}
       else if (menubar_root_p)
 	{
@@ -247,8 +283,7 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
 	  signal_simple_error ("menu name (first element) must be a string",
                                desc);
 	}
-
-      wv->enabled = 1;
+      
       if (deep_p || menubar_root_p)
 	{
 	  widget_value *next;
@@ -321,7 +356,7 @@ menu_item_descriptor_to_widget_value (Lisp_Object desc,
 }
 
 
-#ifdef LWLIB_MENUBARS_LUCID
+#if defined LWLIB_MENUBARS_LUCID || defined LWLIB_MENUBARS_MOTIF
 int in_menu_callback;
 
 static Lisp_Object
@@ -330,7 +365,34 @@ restore_in_menu_callback (Lisp_Object val)
     in_menu_callback = XINT(val);
     return Qnil;
 }
-#endif /* LWLIB_MENUBARS_LUCID */
+#endif /* LWLIB_MENUBARS_LUCID || LWLIB_MENUBARS_MOTIF */
+
+#if 0
+/* #### Sort of a hack needed to process Vactivate_menubar_hook
+   correctly wrt buffer-local values.  A correct solution would
+   involve adding a callback mechanism to run_hook().  This function
+   is currently unused.  */
+static int
+my_run_hook (Lisp_Object hooksym, int allow_global_p)
+{
+  /* This function can GC */
+  Lisp_Object tail;
+  Lisp_Object value = Fsymbol_value (hooksym);
+  int changes = 0;
+
+  if (!NILP (value) && (!CONSP (value) || EQ (XCAR (value), Qlambda)))
+    return !EQ (call0 (value), Qt);
+
+  EXTERNAL_LIST_LOOP (tail, value)
+    {
+      if (allow_global_p && EQ (XCAR (tail), Qt))
+	changes |= my_run_hook (Fdefault_value (hooksym), 0);
+      if (!EQ (call0 (XCAR (tail)), Qt))
+	changes = 1;
+    }
+  return changes;
+}
+#endif
 
 
 /* The order in which callbacks are run is funny to say the least.
@@ -357,12 +419,9 @@ static void
 pre_activate_callback (Widget widget, LWLIB_ID id, XtPointer client_data)
 {
   /* This function can GC */
-  struct gcpro gcpro1;
   struct device *d = get_device_from_display (XtDisplay (widget));
   struct frame *f = x_any_window_to_frame (d, XtWindow (widget));
-  Lisp_Object rest = Qnil;
   Lisp_Object frame;
-  int any_changes = 0;
   int count;
 
   /* set in lwlib to the time stamp associated with the most recent menu
@@ -417,24 +476,17 @@ pre_activate_callback (Widget widget, LWLIB_ID id, XtPointer client_data)
       replace_widget_value_tree (hack_wv, wv->contents);
       free_popup_widget_value_tree (wv);
     }
+  else if (!POPUP_DATAP (FRAME_MENUBAR_DATA (f)))
+    return;
   else
     {
-      if (!POPUP_DATAP (FRAME_MENUBAR_DATA (f)))
-	return;
+#if 0 /* Unused, see comment below. */
+      int any_changes;
+
       /* #### - this menubar update mechanism is expensively anti-social and
 	 the activate-menubar-hook is now mostly obsolete. */
-      /* make the activate-menubar-hook be a list of functions, not a single
-	 function, just to simplify things. */
-      if (!NILP (Vactivate_menubar_hook) &&
-	  (!CONSP (Vactivate_menubar_hook) ||
-	   EQ (XCAR (Vactivate_menubar_hook), Qlambda)))
-	Vactivate_menubar_hook = Fcons (Vactivate_menubar_hook, Qnil);
+      any_changes = my_run_hook (Qactivate_menubar_hook, 1);
 
-      GCPRO1 (rest);
-      for (rest = Vactivate_menubar_hook; !NILP (rest); rest = Fcdr (rest))
-	if (!EQ (call0 (XCAR (rest)), Qt))
-	  any_changes = 1;
-#if 0
       /* #### - It is necessary to *ALWAYS* call set_frame_menubar() now that
 	 incremental menus are implemented.  If a subtree of a menu has been
 	 updated incrementally (a destructive operation), then that subtree
@@ -445,37 +497,17 @@ pre_activate_callback (Widget widget, LWLIB_ID id, XtPointer client_data)
 	 that an INCREMENTAL_TYPE widget_value can be recreated...  Hmmmmm. */
       if (any_changes ||
 	  !XFRAME_MENUBAR_DATA (f)->menubar_contents_up_to_date)
-#endif
 	set_frame_menubar (f, 1, 0);
+#else
+      run_hook (Qactivate_menubar_hook);
+      set_frame_menubar (f, 1, 0);
+#endif
       DEVICE_X_MOUSE_TIMESTAMP (XDEVICE (FRAME_DEVICE (f))) =
 	DEVICE_X_GLOBAL_MOUSE_TIMESTAMP (XDEVICE (FRAME_DEVICE (f))) =
 	x_focus_timestamp_really_sucks_fix_me_better;
-      UNGCPRO;
     }
 }
 
-#ifdef ENERGIZE
-extern int *get_psheets_for_buffer (Lisp_Object, int *);
-
-static void
-set_panel_button_sensitivity (struct frame *f, widget_value *data)
-{
-  struct window *window = XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f));
-  int current_buffer_psheets_count = 0;
-  int *current_buffer_psheets =
-    get_psheets_for_buffer (window->buffer, &current_buffer_psheets_count);
-  int panel_enabled = FRAME_X_DESIRED_PSHEETS (f) ||
-    current_buffer_psheets_count;
-  widget_value *val;
-  for (val = data->contents; val; val = val->next)
-    if (val->name && !strcmp (val->name, "sheet"))
-      {
-	val->enabled = panel_enabled;
-	return;
-      }
-}
-#endif /* ENERGIZE */
-
 static widget_value *
 compute_menubar_data (struct frame *f, Lisp_Object menubar, int deep_p)
 {
@@ -493,10 +525,6 @@ compute_menubar_data (struct frame *f, Lisp_Object menubar, int deep_p)
       Fset_buffer ( XWINDOW (FRAME_SELECTED_WINDOW (f))->buffer);
       data = menu_item_descriptor_to_widget_value (menubar, MENUBAR_TYPE,
 						   deep_p, 0);
-#ifdef ENERGIZE
-      if (data)
-	set_panel_button_sensitivity (f, data);
-#endif
       Fset_buffer (old_buffer);
       unbind_to (count, Qnil);
     }
@@ -678,12 +706,6 @@ make_dummy_xbutton_event (XEvent *dummy,
 }
 
 
-#ifdef ENERGIZE
-extern int desired_debuggerpanel_exposed_p;
-extern int current_debuggerpanel_exposed_p;
-extern int debuggerpanel_sheet;
-extern void notify_energize_sheet_hidden (unsigned long);
-#endif
 
 static void
 x_update_frame_menubar_internal (struct frame *f)
@@ -701,142 +723,20 @@ x_update_frame_menubar_internal (struct frame *f)
   Boolean menubar_was_visible = XtIsManaged (FRAME_X_MENUBAR_WIDGET (f));
   Boolean menubar_will_be_visible = menubar_was_visible;
   Boolean menubar_visibility_changed;
-  Cardinal new_num_top_widgets = 1; /* for the menubar */
-  Widget container = FRAME_X_CONTAINER_WIDGET (f);
-
-#ifdef ENERGIZE
-  int *old_sheets = FRAME_X_CURRENT_PSHEETS (f);
-  int *new_sheets = FRAME_X_DESIRED_PSHEETS (f);
-  int old_count = FRAME_X_CURRENT_PSHEET_COUNT (f);
-  int new_count = FRAME_X_DESIRED_PSHEET_COUNT (f);
-  Lisp_Object old_buf = FRAME_X_CURRENT_PSHEET_BUFFER (f);
-  Lisp_Object new_buf = FRAME_X_DESIRED_PSHEET_BUFFER (f);
-  int psheets_changed = (old_sheets != new_sheets
-			 || old_count != new_count
-			 || !EQ (old_buf, new_buf));
-  int debuggerpanel_changed = (desired_debuggerpanel_exposed_p
-			       != current_debuggerpanel_exposed_p);
-
-  if (desired_debuggerpanel_exposed_p && FRAME_X_TOP_WIDGETS (f) [1] == 0)
-    /* This happens when the frame was just created. */
-    debuggerpanel_changed = 1;
-
-  FRAME_X_CURRENT_PSHEETS (f) = FRAME_X_DESIRED_PSHEETS (f);
-  FRAME_X_CURRENT_PSHEET_COUNT (f) = FRAME_X_DESIRED_PSHEET_COUNT (f);
-  FRAME_X_CURRENT_PSHEET_BUFFER (f) = FRAME_X_DESIRED_PSHEET_BUFFER (f);
-#endif /* ENERGIZE */
 
   if (menubar_contents_changed)
     menubar_will_be_visible = set_frame_menubar (f, 0, 0);
 
   menubar_visibility_changed = menubar_was_visible != menubar_will_be_visible;
 
-  if (! (menubar_visibility_changed
-#ifdef ENERGIZE
-	 || psheets_changed || debuggerpanel_changed
-#endif
-	 ))
+  if (!menubar_visibility_changed)
     return;
 
-
   /* Set menubar visibility */
-  if (menubar_visibility_changed)
-    (menubar_will_be_visible ? XtManageChild : XtUnmanageChild)
-      (FRAME_X_MENUBAR_WIDGET (f));
+  (menubar_will_be_visible ? XtManageChild : XtUnmanageChild)
+    (FRAME_X_MENUBAR_WIDGET (f));
 
-
-#ifdef ENERGIZE
-  /* Set debugger panel visibility */
-  if (debuggerpanel_changed)
-    {
-      Widget w;
-      int sheet = debuggerpanel_sheet;
-
-      w = lw_get_widget (sheet, container, 0);
-      if (desired_debuggerpanel_exposed_p)
-	{
-	  if (! w)
-	    w = lw_make_widget (sheet, container, 0);
-	  FRAME_X_TOP_WIDGETS (f)[1] = w;
-	  XtManageChild (w);
-	}
-      else
-	{
-	  notify_energize_sheet_hidden (sheet);
-	  if (w)
-	    XtUnmanageChild (w);
-	}
-    }
-
-  /* Set psheet visibility.  For the moment we just unmanage all the old
-   ones, and then manage all the new ones.  If the number of psheets
-   ever becomes a large number (i.e. > 1), then we can worry about a
-   more sophisticated way of doing this. */
-  if (psheets_changed)
-    {
-      int i;
-      Widget w;
-      unsigned long sheet;
-
-      for (i=0; i<old_count; i++)
-	{
-	  sheet = old_sheets[i];
-	  w = lw_get_widget (sheet, container, 0);
-	  notify_energize_sheet_hidden (sheet);
-	  if (w)
-	    XtUnmanageChild (w);
-	}
-
-      for (i=0; i<new_count; i++)
-	{
-	  sheet = new_sheets[i];
-	  /* #### This unconditional call to lw_make_widget() is a bad
-	     idea.  Doesn't it cause a memory leak if the widget
-	     already exists?
-
-	     #### How does Energize know that a sheet just got displayed?
-	     #### Energize knows all.  */
-	  w = lw_make_widget (sheet, container, 0);
-	  FRAME_X_TOP_WIDGETS (f)[2+i] = w;
-	  XtManageChild (w);
-	}
-    }
-
-  new_num_top_widgets += 1+new_count;
-#endif /* ENERGIZE */
-
-  /* Note that new_num_top_widgets doesn't need to reflect the actual
-     number of top widgets, but just the limit of FRAME_X_TOP_WIDGETS (f)[]. */
-  FRAME_X_NUM_TOP_WIDGETS (f) = new_num_top_widgets;
-  {
-    /* We want to end up as close in size as possible to what we
-       were before.  So, ask the EmacsManager what size it wants to be
-       (suggesting the current size), and resize it to that size.  It
-       in turn will call our query-geometry callback, which will round
-       the size to something that exactly fits the text widget. */
-    XtWidgetGeometry req, repl;
-    Arg al [2];
-
-    req.request_mode = CWWidth | CWHeight;
-    XtSetArg (al [0], XtNwidth,  &req.width);
-    XtSetArg (al [1], XtNheight, &req.height);
-    XtGetValues (container, al, 2);
-    XtQueryGeometry (container, &req, &repl);
-    EmacsManagerChangeSize (container, repl.width, repl.height);
-    /* The window size might not have changed but the text size
-       did; thus, the base size might be incorrect.  So update it. */
-    EmacsShellUpdateSizeHints (FRAME_X_SHELL_WIDGET (f));
-  }
-
-#ifdef ENERGIZE
-  /* Give back the focus to emacs if no psheets are displayed anymore */
-  if (psheets_changed)
-    {
-      Lisp_Object frame;
-      XSETFRAME (frame, f);
-      Fselect_frame (frame);
-    }
-#endif /* ENERGIZE */
+  MARK_FRAME_SIZE_SLIPPED (f);
 }
 
 static void
@@ -848,9 +748,6 @@ x_update_frame_menubars (struct frame *f)
 
   /* #### This isn't going to work right now that this function works on
      a per-frame, not per-device basis.  Guess what?  I don't care. */
-#ifdef ENERGIZE
-  current_debuggerpanel_exposed_p = desired_debuggerpanel_exposed_p;
-#endif
 }
 
 static void
@@ -867,43 +764,6 @@ x_free_frame_menubars (struct frame *f)
       lw_destroy_all_widgets (id);
       XFRAME_MENUBAR_DATA (f)->id = 0;
     }
-
-#ifdef ENERGIZE
-  {
-    /* Also destroy this frame's psheets */
-    Widget parent = FRAME_X_CONTAINER_WIDGET (f);
-    int *sheets = FRAME_X_CURRENT_PSHEETS (f);
-    int i = FRAME_X_CURRENT_PSHEET_COUNT (f);
-    while (i--)
-      {
-	unsigned long sheet = sheets [i];
-	Widget w = lw_get_widget (sheet, parent, 0);
-	if (w)
-	  lw_destroy_widget (w);
-      }
-    FRAME_X_CURRENT_PSHEET_COUNT (f) = 0;
-
-    /* Is this necessary? */
-    sheets = FRAME_X_DESIRED_PSHEETS (f);
-    i = FRAME_X_DESIRED_PSHEET_COUNT (f);
-    while (i--)
-      {
-	unsigned long sheet = sheets [i];
-	Widget w = lw_get_widget (sheet, parent, 0);
-	if (w)
-	  lw_destroy_widget (w);
-      }
-    FRAME_X_DESIRED_PSHEET_COUNT (f) = 0;
-
-    /* sigh... debugger panel is special... */
-    if (debuggerpanel_sheet)
-      {
-	Widget w = lw_get_widget (debuggerpanel_sheet, parent, 0);
-	if (w)
-	  lw_destroy_widget (w);
-      }
-  }
-#endif /* ENERGIZE */
 }
 
 static void
@@ -916,7 +776,7 @@ x_popup_menu (Lisp_Object menu_desc, Lisp_Object event)
   Widget menu;
   struct Lisp_Event *eev = NULL;
   XEvent xev;
-  Lisp_Object frame = Qnil;
+  Lisp_Object frame;
 
   XSETFRAME (frame, f);
   CHECK_X_FRAME (frame);
@@ -996,7 +856,7 @@ console_type_create_menubar_x (void)
 void
 vars_of_menubar_x (void)
 {
-  last_popup_menu_selection_callback_id = -1;
+  last_popup_menu_selection_callback_id = (LWLIB_ID) -1;
 
 #if defined (LWLIB_MENUBARS_LUCID)
   Fprovide (intern ("lucid-menubars"));
