@@ -214,8 +214,8 @@ scmp_1 (CONST Bufbyte *s1, CONST Bufbyte *s2, Charcount len,
     {
       while (l)
         {
-          Bufbyte c1 = DOWNCASE (current_buffer, charptr_emchar (s1));
-          Bufbyte c2 = DOWNCASE (current_buffer, charptr_emchar (s2));
+          Emchar c1 = DOWNCASE (current_buffer, charptr_emchar (s1));
+          Emchar c2 = DOWNCASE (current_buffer, charptr_emchar (s2));
 
           if (c1 == c2)
             {
@@ -252,14 +252,12 @@ regexp_ignore_completion_p (CONST Bufbyte *nonreloc,
   if (!NILP (Vcompletion_regexp_list))
     {
       Lisp_Object regexps;
-      for (regexps = Vcompletion_regexp_list;
-	   CONSP (regexps);
-	   regexps = XCDR (regexps))
+      EXTERNAL_LIST_LOOP (regexps, Vcompletion_regexp_list)
 	{
 	  Lisp_Object re = XCAR (regexps);
-	  if (STRINGP (re)
-	      && (fast_string_match (re, nonreloc, reloc, offset,
-				     length, 0, ERROR_ME, 0) < 0))
+	  CHECK_STRING (re);
+	  if (fast_string_match (re, nonreloc, reloc, offset,
+				 length, 0, ERROR_ME, 0) < 0)
 	    return 1;
 	}
     }
@@ -291,7 +289,16 @@ ignore_completion_p (Lisp_Object completion_string,
 }
 
 
+/* #### Maybe we should allow ALIST to be a hashtable.  It is wrong
+   for the use of obarrays to be better-rewarded than the use of
+   hashtables.  By better-rewarded I mean that you can pass an obarray
+   to all of the completion functions, whereas you can't do anything
+   like that with a hashtable.
 
+   To do so, there should probably be a
+   map_obarray_or_alist_or_hashtable function which would be used by
+   both Ftry_completion and Fall_completions.  But would the
+   additional funcalls slow things down?  */
 
 DEFUN ("try-completion", Ftry_completion, 2, 3, 0, /*
 Return common substring of all completions of STRING in ALIST.
@@ -328,13 +335,13 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
   CHECK_STRING (string);
 
   if (CONSP (alist))
-  {
-    Lisp_Object tem = XCAR (alist);
-    if (SYMBOLP (tem))          /* lambda, autoload, etc.  Emacs-lisp sucks */
-      return call3 (alist, string, pred, Qnil);
-    else
-      list = 1;
-  }
+    {
+      Lisp_Object tem = XCAR (alist);
+      if (SYMBOLP (tem))	/* lambda, autoload, etc.  Emacs-lisp sucks */
+	return call3 (alist, string, pred, Qnil);
+      else
+	list = 1;
+    }
   else if (VECTORP (alist))
     list = 0;
   else if (NILP (alist))
@@ -344,7 +351,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 
   bestmatch = Qnil;
   blength = 0;
-  slength = string_char_length (XSTRING (string));
+  slength = XSTRING_CHAR_LENGTH (string);
 
   /* If ALIST is not a list, set TAIL just for gc pro.  */
   tail = alist;
@@ -381,10 +388,11 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	  if (!ZEROP (bucket))
 	    {
               struct Lisp_Symbol *next;
-	      if (!SYMBOLP (bucket)) {
-		signal_simple_error("Bad obarry passed to try-completions",
-				    bucket);
-	      }
+	      if (!SYMBOLP (bucket))
+		{
+		  signal_simple_error ("Bad obarray passed to try-completions",
+				       bucket);
+		}
 	      next = symbol_next (XSYMBOL (bucket));
 	      elt = bucket;
 	      eltstring = Fsymbol_name (elt);
@@ -406,7 +414,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 
       if (STRINGP (eltstring))
 	{
-	  Charcount eltlength = string_char_length (XSTRING (eltstring));
+	  Charcount eltlength = XSTRING_CHAR_LENGTH (eltstring);
 	  if (slength <= eltlength
 	      && (0 > scmp (XSTRING_DATA (eltstring),
                             XSTRING_DATA (string),
@@ -531,13 +539,13 @@ the symbol from the obarray.
   CHECK_STRING (string);
 
   if (CONSP (alist))
-  {
-    Lisp_Object tem = XCAR (alist);
-    if (SYMBOLP (tem))          /* lambda, autoload, etc.  Emacs-lisp sucks */
-      return call3 (alist, string, pred, Qt);
-    else
-      list = 1;
-  }
+    {
+      Lisp_Object tem = XCAR (alist);
+      if (SYMBOLP (tem))	/* lambda, autoload, etc.  Emacs-lisp sucks */
+	return call3 (alist, string, pred, Qt);
+      else
+	list = 1;
+    }
   else if (VECTORP (alist))
     list = 0;
   else if (NILP (alist))
@@ -546,7 +554,7 @@ the symbol from the obarray.
     return call3 (alist, string, pred, Qt);
 
   allmatches = Qnil;
-  slength = string_char_length (XSTRING (string));
+  slength = XSTRING_CHAR_LENGTH (string);
 
   /* If ALIST is not a list, set TAIL just for gc pro.  */
   tail = alist;
@@ -602,10 +610,10 @@ the symbol from the obarray.
       /* Is this element a possible completion? */
 
       if (STRINGP (eltstring)
-          && (slength <= string_char_length (XSTRING (eltstring)))
+          && (slength <= XSTRING_CHAR_LENGTH (eltstring))
 	  /* Reject alternatives that start with space
 	     unless the input starts with space.  */
-	  && ((string_char_length (XSTRING (string)) > 0 &&
+	  && ((XSTRING_CHAR_LENGTH (string) > 0 &&
 	       string_char (XSTRING (string), 0) == ' ')
 	      || string_char (XSTRING (eltstring), 0) != ' ')
           && (0 > scmp (XSTRING_DATA (eltstring),
