@@ -29,36 +29,13 @@ Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_TOOLBARS
 
-/* There are 4 of these per frame.  They don't really need to be an
-   lrecord (they're not lisp-accessible) but it makes marking slightly
-   more modular.  */
-struct toolbar_data
-{
-  struct lcrecord_header header;
+#define FRAME_TOOLBAR_BUTTONS(frame, pos)	\
+  ((frame)->toolbar_buttons[pos])
+#define FRAME_CURRENT_TOOLBAR_SIZE(frame, pos)	\
+  ((frame)->current_toolbar_size[pos])
+#define DEVICE_SUPPORTS_TOOLBARS_P(d)		\
+  (HAS_DEVMETH_P ((d), output_frame_toolbars))
 
-  /* The last buffer for which the toolbars were displayed. */
-  Lisp_Object last_toolbar_buffer;
-
-  /* The actual buttons are chained through this. */
-  Lisp_Object toolbar_buttons;
-};
-
-DECLARE_LRECORD (toolbar_data, struct toolbar_data);
-#define XTOOLBAR_DATA(x) XRECORD (x, toolbar_data, struct toolbar_data)
-#define XSETTOOLBAR_DATA(x, p) XSETRECORD (x, p, toolbar_data)
-#define TOOLBAR_DATAP(x) RECORDP (x, toolbar_data)
-#define GC_TOOLBAR_DATAP(x) GC_RECORDP (x, toolbar_data)
-#define CHECK_TOOLBAR_DATA(x) CHECK_RECORD (x, toolbar_data)
-#define CONCHECK_TOOLBAR_DATA(x) CONCHECK_RECORD (x, toolbar_data)
-
-#define FRAME_TOOLBAR_DATA(frame, position)				\
-  (XTOOLBAR_DATA ((frame)->toolbar_data[position]))
-#define FRAME_TOOLBAR_BUFFER(frame, position)				 \
-  (XTOOLBAR_DATA ((frame)->toolbar_data[position])->last_toolbar_buffer)
-
-/* These are chained together through toolbar_buttons in struct
-   toolbar_data.  These don't need to be an lrecord either, but again,
-   it makes marking easier. */
 struct toolbar_button
 {
   struct lcrecord_header header;
@@ -86,6 +63,10 @@ struct toolbar_button
   int x, y;
   int width, height;
   int dirty;
+  /* is this button in a left or right toolbar? */
+  int vertical;
+  /* border_width when this button was layed out */
+  int border_width;
 };
 
 DECLARE_LRECORD (toolbar_button, struct toolbar_button);
@@ -96,11 +77,11 @@ DECLARE_LRECORD (toolbar_button, struct toolbar_button);
 #define CHECK_TOOLBAR_BUTTON(x) CHECK_RECORD (x, toolbar_button)
 #define CONCHECK_TOOLBAR_BUTTON(x) CONCHECK_RECORD (x, toolbar_button)
 
-extern void get_toolbar_coords (struct frame *f, enum toolbar_pos pos, int *x,
-				int *y, int *width, int *height, int *vert,
-				int for_layout);
-extern Lisp_Object toolbar_button_at_pixpos (struct frame *f, int x_coord,
-					     int y_coord);
+void get_toolbar_coords (struct frame *f, enum toolbar_pos pos, int *x,
+			 int *y, int *width, int *height, int *vert,
+			 int for_layout);
+Lisp_Object toolbar_button_at_pixpos (struct frame *f, int x_coord,
+				      int y_coord);
 DECLARE_SPECIFIER_TYPE (toolbar);
 #define XTOOLBAR_SPECIFIER(x) XSPECIFIER_TYPE (x, toolbar)
 #define XSETTOOLBAR_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, toolbar)
@@ -108,12 +89,17 @@ DECLARE_SPECIFIER_TYPE (toolbar);
 #define CHECK_TOOLBAR_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, toolbar)
 #define CONCHECK_TOOLBAR_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, toolbar)
 
+#define MSWINDOWS_DEFAULT_TOOLBAR_HEIGHT	37
+#define MSWINDOWS_DEFAULT_TOOLBAR_WIDTH		40
+#define MSWINDOWS_DEFAULT_TOOLBAR_BORDER_WIDTH	0
 #define DEFAULT_TOOLBAR_HEIGHT	37
-#define DEFAULT_TOOLBAR_WIDTH	40
+#define DEFAULT_TOOLBAR_WIDTH		40
 #define DEFAULT_TOOLBAR_BLANK_SIZE	8
-#define MINIMUM_SHADOW_THICKNESS	2
+#define DEFAULT_TOOLBAR_BORDER_WIDTH	0
+#define MINIMUM_SHADOW_THICKNESS	1
 
 extern Lisp_Object Vtoolbar_size[4];
+extern Lisp_Object Vtoolbar_border_width[4];
 void update_frame_toolbars (struct frame *f);
 void init_frame_toolbars (struct frame *f);
 void init_device_toolbars (struct device *d);
