@@ -1,8 +1,8 @@
 #!/bin/sh
 emacs_major_version=21
 emacs_minor_version=1
-emacs_beta_version=4
-xemacs_codename="Arches"
+emacs_beta_version=6
+xemacs_codename="Big Bend"
 infodock_major_version=4
 infodock_minor_version=0
 infodock_build_version=7
